# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl.block_store.v2 import availability_zone as _availability_zone
from ecl.block_store.v2 import extension as _extension
from ecl.block_store.v2 import limit as _limit
from ecl.block_store.v2 import metadata as _metadata
from ecl.block_store.v2 import quota as _quota
from ecl.block_store.v2 import type as _type
from ecl.block_store.v2 import volume as _volume

from ecl import proxy


class Proxy(proxy.BaseProxy):

    def types(self):
        """Return a list of types

        :returns: A list of Type objects
        :rtype: :class:`~ecl.block_store.v2.type.Type`
        """
        return list(self._list(_type.Type, paginated=False))

    def find_volume_type(self, name_or_id, ignore_missing=False):
        """Find a volume type

        :param name_or_id: The name or ID of a volume type.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :returns: One :class:`~ecl.block_store.v2.type.Type` or None
        """
        return self._find(_type.Type, name_or_id,
                          ignore_missing=ignore_missing)

    def volumes(self):
        """Return a list of volume

        :returns: A list of Volume objects
        :rtype: :class:`~ecl.block_store.v2.volume.Volume`
        """
        return list(self._list(_volume.Volume, paginated=False))

    def get_volume(self, volume):
        """Get a single volume

        :param volume: The value can be the ID of a volume or a
                       :class:`~ecl.volume.v2.volume.Volume` instance.

        :returns: One :class:`~ecl.volume.v2.volume.Volume`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_volume.Volume, volume)

    def create_volume(self, size, name=None, description=None,
                      availability_zone=None, imageRef=None, metadata=None,
                      snapshot_id=None, volume_type=None, source_volid=None):
        """Create a new volume from attributes

        :param int size: size of volume to create
        :param string name: name of volume to create
        :param string description: description of volume to create
        :param string availability_zone: availability zone of volume to create
        :param string imageRef: image ID of volume to create from
        :param dict metadata: metadata of volume to create
        :param string snapshot_id: snapshot ID of volume to create
        :param string volume_type: type of volume to create
        :param string source_volid: source volume id of volume to create from

        :returns: The results of volume creation
        :rtype: :class:`~ecl.volume.v2.volume.Volume`
        """
        attrs = dict()
        attrs.update({"size": size})
        if name:
            attrs.update({"name": name})
        if description:
            attrs.update({"description": description})
        if availability_zone:
            attrs.update({"availability_zone": availability_zone})
        if imageRef:
            attrs.update({"imageRef": imageRef})
        if metadata:
            attrs.update({"metadata": metadata})
        if snapshot_id:
            attrs.update({"snapshot_id": snapshot_id})
        if volume_type:
            attrs.update({"volume_type": volume_type})
        if source_volid:
            attrs.update({"source_volid": source_volid})
        return self._create(_volume.Volume, **attrs)

    def delete_volume(self, volume, ignore_missing=False):
        """Delete a volume

        :param volume: The value can be either the ID of a volume or a
                       :class:`~ecl.volume.v2.volume.Volume` instance.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the volume does not exist.
                    When set to ``True``, no exception will be set when
                    attempting to delete a nonexistent volume.

        :returns: ``None``
        """
        self._delete(_volume.Volume, volume, ignore_missing=ignore_missing)

    def update_volume(self, volume, **body):
        """Create a new volume from attributes

        :param volume: The value can be the ID of a volume or a
                       :class:`~ecl.volume.v2.volume.Volume` instance.
        :param string name: name of volume to update
        :param string description: description of volume to update

        :returns: The results of volume update
        :rtype: :class:`~ecl.volume.v2.volume.Volume`
        """
        self._update(_volume.Volume, volume, **body)

    def extend_volume(self, volume, new_size):
        """Extend size of a volume

        :param volume: The value can be the ID of a volume or a
                       :class:`~ecl.volume.v2.volume.Volume` instance.
        :param string new_size: size of volume to extend

        :returns: The results of volume extend size
        :rtype: :class:`~ecl.volume.v2.volume.Volume`
        """
        volume = self._get_resource(_volume.Volume, volume)
        return volume.extend_size(self.session, new_size)

    def upload_to_image(self, volume, container_format=None,
                        disk_format=None, image_name=None,
                        force=False):
        """

        :param volume: The value can be the ID of a volume or a
                       :class:`~ecl.volume.v2.volume.Volume` instance.
        :param string container_format: Select one of the following
                        (bare, ovf, ami, aki, ari)
        :param string disk_format: Select one of the following
                        (raw, vmdk, vdi, qcow2)
        :param string image_name: Name of image to upload
        :param boolean force: Whether to upload a volume of attached state.
                        When the state of volume is available,
                        the value of force option does not affect the operation.
        :return: Object describes detail of volume_upload_image
        """
        volume = self._get_resource(_volume.Volume, volume)
        return volume.upload_to_image(self.session,
                                      container_format=container_format,
                                      disk_format=disk_format,
                                      image_name=image_name,
                                      force=force)

    def update_bootable(self, volume, bootable=False):
        """

        :param volume: The value can be the ID of a volume or a
                       :class:`~ecl.volume.v2.volume.Volume` instance.
        :param boolean bootable: bootable status to update
        :return: None
        """
        volume = self._get_resource(_volume.Volume, volume)
        return volume.update_bootable(self.session, bootable)

    def availability_zones(self):
        """Return a list of availability zones

        :returns: A list of availability zone
        :rtype: :class:`~ecl.block_store.v2.availability_zone.AvailabilityZone`
        """
        return list(self._list(_availability_zone.AvailabilityZone, paginated=False))

    def find_availability_zone(self, name_or_id, ignore_missing=False):
        """Find a single availability_zone

        :param name_or_id: The name or ID of a availability_zone.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :returns: One :class:`~ecl.block_store.v2.availability_zone.AvailabilityZone` or None
        """
        return self._find(_availability_zone.AvailabilityZone, name_or_id,
                          ignore_missing=ignore_missing)

    def get_quota(self, tenant_id):
        """Get quota info of a tenant

        :param tenant_id: The ID for the tenant for which you want to show
                        quotas. This ID is different from the tenant ID of authentication.
                        That ID is for the admin tenant.

        :returns: One :class:`~ecl.block_store.v2.quota.Quota`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_quota.Quota, tenant_id)

    def get_quota_detail(self, tenant_id, user_id):
        """Get quota info of a tenant

        :param tenant_id: The ID for the tenant for which you want to show
                        quotas. This ID is different from the tenant ID of authentication.
                        That ID is for the admin tenant.
        :param user_id: The user ID. Specify in the URI as a query string

        :returns: One :class:`~ecl.block_store.v2.quota.DetailQuota`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_quota.DetailQuota, user_id,
                         path_args={"tenant_id":tenant_id})

    def get_default_quota(self, tenant_id):
        """Get default quota of a tenant

        :param tenant_id: The ID for the tenant for which you want to show
                        quotas. This ID is different from the tenant ID of authentication.
                        That ID is for the admin tenant.
        :param user_id: The user ID. Specify in the URI as a query string

        :returns: One :class:`~ecl.block_store.v2.quota.DefaultQuota`
        :raises: :class:`~ecl.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_quota.DefaultQuota, None,
                         path_args={"tenant_id": tenant_id})

    def extensions(self):
        """Return a list of Extensions

        :returns: A list of Extension objects
        :rtype: :class:`~ecl.block_store.v2.extension.Extension`
        """
        return list(self._list(_extension.Extension, paginated=False))

    def limits(self):
        """Return a list of Limit

        :returns: show Limit objects
        :rtype: :class:`~ecl.block_store.v2.limit.Limit`
        """
        return self._get(_limit.Limit, None)

    def update_metadata(self, volume_id, **metadata):
        """
        Updates metadata items by key for a specified volume.

        :param volume_id: volume id to update metadata
        :param metadata: metadata to updata
        :return: :class:`~ecl.block_store.v2.metadata.Metadata
        """
        meta = _metadata.Metadata()
        return meta.update(self.session, volume_id, **metadata)

    def create_metadata(self, volume_id, **metadata):
        """
        Creates or replaces metadata items for a specified volume.
        
        :param volume_id: volume id to create metadata
        :param metadata: metadata to create
        :return: :class:`~ecl.block_store.v2.metadata.Metadata
        """
        meta = _metadata.Metadata()
        return meta.create(self.session, volume_id, **metadata)

    def find_volume(self, name_or_id, ignore_missing=False):
        """Find a single volume

        :param name_or_id: The name or ID of a server.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :returns: One :class:`~ecl.compute.v2.volume.Volume` or None
        """
        return self._find(_volume.Volume, name_or_id,
                          ignore_missing=ignore_missing)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from ecl import exceptions
from ecl.image import image_service
from ecl import resource2
from ecl import utils
import json


class Image(resource2.Resource):
    resources_key = 'images'
    service = image_service.ImageService()
    base_path = '/' + service.version + '/images'


    # capabilities
    allow_create = True
    allow_get = True
    allow_update = True
    allow_delete = True
    allow_list = True
    patch_update = True

    _query_mapping = resource2.QueryParameters(
        "limit", "marker",
        "name", "visibility",
        "member_status", "owner",
        "status", "size_min",
        "size_max", "sort_key",
        "sort_dir", "tag"
    )

    # NOTE: Do not add "self" support here. If you've used Python before,
    # you know that self, while not being a reserved word, has special
    # meaning. You can't call a class initializer with the self name
    # as the first argument and then additionally in kwargs, as we
    # do when we're constructing instances from the JSON body.
    # Resource.list explicitly pops off any "self" keys from bodies so
    # that we don't end up getting the following:
    # TypeError: __init__() got multiple values for argument 'self'

    # The image data (bytes or a file-like object)
    data = None

    # Properties
    #: Descriptive name for the image
    name = resource2.Body('name')
    #: Status of the image
    image_status = resource2.Body('status')
    #: String related to the image
    tags = resource2.Body('tags')
    #: The container format refers to whether the VM image is in a file
    #: format that also contains metadata about the actual VM.
    #: Container formats include OVF and Amazon AMI. In addition,
    #: a VM image might not have a container format - instead,
    #: the image is just a blob of unstructured data.
    container_format = resource2.Body('container_format')
    #: Date and time of image registration
    created_at = resource2.Body('created_at')
    #: Valid values are: aki, ari, ami, raw, iso, vhd, vdi, qcow2, or vmdk.
    #: The disk format of a VM image is the format of the underlying
    #: disk image. Virtual appliance vendors have different formats
    #: for laying out the information contained in a VM disk image.
    disk_format = resource2.Body('disk_format')
    #: Include location_url and location_metadata.
    #: location_url: URL to access the image file kept in external store
    #: (it is shown when 'show_multiple_locations' option is enabled)
    locations = resource2.Body('locations')
    #: The URL to access the image file kept in external store. It appears
    #: when you set the show_image_direct_url option to true in the
    #: Image service's configuration file.
    direct_url = resource2.Body('direct_url')
    #: Date and time of the last image modification.
    updated_at = resource2.Body('updated_at')
    #: The image visibility.
    visibility = resource2.Body('visibility')
    #: Amount of disk space (in GB) required to boot image.
    min_disk = resource2.Body('min_disk')
    #: If true, image will not be deletable.
    protected = resource2.Body('protected')
    #: An identifier for the image.
    id = resource2.Body('id')
    #: URL for the virtual machine image file.
    file = resource2.Body('file')
    #: md5 hash of image contents
    checksum = resource2.Body('checksum')
    #: Owner of the image.
    owner = resource2.Body('owner')
    #: Size of image file in bytes.
    size = resource2.Body('size')
    #: Amount of ram (in MB) required to boot image.
    min_ram = resource2.Body('min_ram')
    #: URL for schema of the virtual machine image.
    image_schema = resource2.Body('image_schema')
    ##: URL for the virtual machine image.
    self = resource2.Body('self')

    #: URL for schema of the virtual machine images.
    images_schema = resource2.Body('images_schema')
    #: URL for the first page of response
    first = resource2.Body('first')

    #: The key of the extra properties.
    extra_key = resource2.Body('extra_key')
    #: The value of the extra properties.
    extra_value = resource2.Body('extra_value')
    #: OS information of this image.
    os_type = resource2.Body('.os.type')
    #: Service information of this image.
    service_type = resource2.Body('.service.type')

    # def _get_image_with_extra_key(self, attr):
    #     image = self.existing(**attr)
    #
    #     for key in attr.keys():
    #         if not hasattr(image, key):
    #             image.__setattr__(key, attr[key])
    #     return image
    #
    # def list(self, session, **query):
    #     """Lists virtual machine (VM) images."""
    #
    #     if query != None:
    #         query_params = self._query_mapping._transpose(query)
    #         uri = self.base_path
    #
    #         params = []
    #         for key in query_params.keys():
    #             item = "%s=%s" % (str(key), str(query_params[key]))
    #             params.append(item)
    #
    #         if len(params) != 0:
    #             uri += '?'
    #             for item in params:
    #                 uri += item + '&'
    #             uri = uri[:-1]
    #
    #     resp = session.get(
    #         uri,
    #         headers={"Accept": "application/json"},
    #         endpoint_filter=self.service
    #     )
    #     resp = resp.json()
    #     images_schema = resp['schema']
    #     first = resp['first']
    #     resp = resp[self.resources_key]
    #
    #     for data in resp:
    #         # Do not allow keys called "self" through. Glance chose
    #         # to name a key "self", so we need to pop it out because
    #         # we can't send it through cls.existing and into the
    #         # Resource initializer. "self" is already the first
    #         # argument and is practically a reserved word.
    #         data.pop("self", None)
    #
    #         image = self._get_image_with_extra_key(data)
    #         image.images_schema = images_schema
    #         image.first = first
    #
    #         yield image

    def get(self, session, image_id):
        """Get a single image's detailed information."""
        url = utils.urljoin(self.base_path, image_id)
        resp = session.get(
            url,
            headers={"Accept": "application/json"},
            endpoint_filter=self.service
        )
        #For extra key/value use.
        self.response = resp.json()
        self._translate_response(resp, has_body=True)
        return self

    def update(self, session, image_id, attrs_list):
        """Updates a specified image."""
        url = utils.urljoin(self.base_path, image_id)
        attrs_list = json.dumps(attrs_list)

        resp = session.patch(url, endpoint_filter=self.service, data=attrs_list, headers={"Content-Type": "application/openstack-images-v2.1-json-patch", "Accept": ""})
        self._translate_response(resp, has_body=True)
        return self

    def upload(self, session, image_id, image_data):
        """Upload data into an existing image"""
        url = utils.urljoin(self.base_path, image_id, 'file')
        resp = session.put(url, endpoint_filter=self.service,
                    data=image_data,
                    headers={"Content-Type": "application/octet-stream",
                             "Accept": "application/json"})
        self._translate_response(resp, has_body=False)
        return self

    def download(self, session, image_id):
        """Download the data contained in an image"""
        # TODO(briancurtin): This method should probably offload the get
        # operation into another thread or something of that nature.
        url = utils.urljoin(self.base_path, image_id, 'file')
        resp = session.get(url, endpoint_filter=self.service)
        return resp.content

    @classmethod
    def find(cls, session, name_or_id, ignore_missing=False, **params):
        """Find a resource by its name or id.

        :param session: The session to use for making this request.
        :type session: :class:`~ecl.session.Session`
        :param name_or_id: This resource's identifier, if needed by
                           the request. The default is ``None``.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :param dict params: Any additional parameters to be passed into
                            underlying methods, such as to
                            :meth:`~ecl.resource2.Resource.existing`
                            in order to pass on URI parameters.

        :return: The :class:`Resource` object matching the given name or id
                 or None if nothing matches.
        :raises: :class:`ecl.exceptions.DuplicateResource` if more
                 than one resource is found for this request.
        :raises: :class:`ecl.exceptions.ResourceNotFound` if nothing
                 is found and ignore_missing is ``False``.
        """
        # Try to short-circuit by looking directly for a matching ID.
        params["limit"] = 10000
        data = cls.list(session, **params)

        result = cls._get_one_match(name_or_id, data)
        if result is not None:
            return result

        if ignore_missing:
            return None
        raise exceptions.ResourceNotFound(
            "No %s found for %s" % (cls.__name__, name_or_id))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl.database import database_service
from ecl import resource2


class Datastore(resource2.Resource):
    resource_key = None
    resources_key = 'datastores'
    base_path = '/datastores'
    service = database_service.DatabaseService()

    # capabilities
    allow_list = True

    _query_mapping = resource2.QueryParameters()

    # Properties
    #: The ID of this datastore
    id = resource2.Body('id')
    #: The name of this datastore.
    name = resource2.Body('name')
    #: Size of the disk this datastore offers. *Type: int*
    default_version = resource2.Body('default_version')
    #: The amount of RAM (in MB) this datastore offers. *Type: int*
    versions = resource2.Body('versions')
    #: Links pertaining to this datastore. This is a list of dictionaries,
    #: each including keys ``href`` and ``rel``.
    links = resource2.Body('links')



# Licensed under the Apache License, Version 2.0 (the "License"); you may

# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl.object_store.v1 import _base
from ecl import resource


class Account(_base.BaseResource):
    _custom_metadata_prefix = "X-Account-Meta-"

    base_path = "/"

    allow_retrieve = True
    allow_update = True
    allow_head = True

    #: The total number of bytes that are stored in Object Storage for
    #: the account.
    account_bytes_used = resource.header("x-account-bytes-used", type=int)
    #: The number of containers.
    account_container_count = resource.header("x-account-container-count",
                                              type=int)
    #: The number of objects in the account.
    account_object_count = resource.header("x-account-object-count", type=int)
    #: The secret key value for temporary URLs. If not set,
    #: this header is not returned by this operation.
    meta_temp_url_key = resource.header("x-account-meta-temp-url-key")
    #: A second secret key value for temporary URLs. If not set,
    #: this header is not returned by this operation.
    meta_temp_url_key_2 = resource.header("x-account-meta-temp-url-key-2")
    #: The timestamp of the transaction.
    timestamp = resource.header("x-timestamp")



# -*- coding: utf-8 -*-

import testtools

from ecl.provider_connectivity.v1 import tenant_connection


IDENTIFIER = 'IDENTIFIER'
REQUEST_ID = 'REQUEST_ID'
BASIC_EXAMPLE = {
    'id': IDENTIFIER,
    'tenant_connection_id': IDENTIFIER,
    'tenant_connection_request_id': REQUEST_ID,
    'name': 'name',
    'description': 'description',
    'tags': {'key1': 'value2'},
    'tenant_id': 'tenant_id',
    'name_other': 'name_other',
    'description_other': 'description_other',
    'tags_other': 'tags_other',
    'tenant_id_other': 'tenant_id_other',
    'network_id': 'network_id',
    'device_type': 'device_type',
    'device_id': 'device_id',
    'device_interface_id': 'device_interface_id',
    'attachment_opts': {
      "segmentation_type": "segmentation_type",
      "segmentation_id": "segmentation_id",
      "fixed_ip": []
    },
    'port_id': 'port_id',
    'status': 'status'
}


class TestTenantConnection(testtools.TestCase):
    def test_basic(self):
        sot = tenant_connection.TenantConnection()
        self.assertEqual('tenant_connections', sot.resources_key)
        self.assertEqual('tenant_connection', sot.resource_key)
        self.assertEqual('/v2.0/tenant_connections', sot.base_path)
        self.assertEqual('provider-connectivity', sot.service.service_type)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_get)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_update)

    def test_make_basic(self):
        sot = tenant_connection.TenantConnection(**BASIC_EXAMPLE)
        self.assertEqual(BASIC_EXAMPLE['id'], sot.id)
        self.assertEqual(BASIC_EXAMPLE['tenant_connection_id'], sot.tenant_connection_id)
        self.assertEqual(BASIC_EXAMPLE['tenant_connection_request_id'], sot.tenant_connection_request_id)
        self.assertEqual(BASIC_EXAMPLE['name'], sot.name)
        self.assertEqual(BASIC_EXAMPLE['description'], sot.description)
        self.assertEqual(BASIC_EXAMPLE['tags'], sot.tags)
        self.assertEqual(BASIC_EXAMPLE['tenant_id'], sot.tenant_id)
        self.assertEqual(BASIC_EXAMPLE['name_other'], sot.name_other)
        self.assertEqual(BASIC_EXAMPLE['description_other'], sot.description_other)
        self.assertEqual(BASIC_EXAMPLE['tags_other'], sot.tags_other)
        self.assertEqual(BASIC_EXAMPLE['tenant_id_other'], sot.tenant_id_other)
        self.assertEqual(BASIC_EXAMPLE['network_id'], sot.network_id)
        self.assertEqual(BASIC_EXAMPLE['device_type'], sot.device_type)
        self.assertEqual(BASIC_EXAMPLE['device_id'], sot.device_id)
        self.assertEqual(BASIC_EXAMPLE['device_interface_id'], sot.device_interface_id)
        self.assertEqual(BASIC_EXAMPLE['attachment_opts'], sot.attachment_opts)
        self.assertEqual(BASIC_EXAMPLE['port_id'], sot.port_id)
        self.assertEqual(BASIC_EXAMPLE['status'], sot.status)



# -*- coding: utf-8 -*-

import testtools

from ecl.connectivity.v1.mcic import MCIC

IDENTIFIER = 'IDENTIFIER'
BASIC_EXAMPLE = {
    'mcic_id': IDENTIFIER,
    'mcic_name': 'xxx-name',
    'mcic_status': 'xxx-status',
    'tenant_id': 'xxx-tenant-id',
    'tenant_name': 'xxx-tenant-name',
    'service_type': 'xxx-service-type',
    'ngc': None,
    'colo': {},
    'timezone': 'UTC'
}


class TestMCIC(testtools.TestCase):

    def test_basic(self):
        sot = MCIC()
        self.assertEqual(None, sot.resource_key)
        self.assertEqual(None, sot.resources_key)
        self.assertEqual('/mCICs', sot.base_path)
        self.assertEqual('interconnectivity', sot.service.service_type)
        self.assertFalse(sot.allow_create)
        self.assertTrue(sot.allow_get)
        self.assertFalse(sot.allow_delete)
        self.assertTrue(sot.allow_list)
        self.assertFalse(sot.allow_update)

        self.assertDictEqual({"sort_key": "sort_key",
                              "sort_dir": "sort_dir"
                              },
                             sot._query_mapping._mapping)

    def test_make_basic(self):
        sot = MCIC(**BASIC_EXAMPLE)
        self.assertEqual(BASIC_EXAMPLE['mcic_id'], sot.mcic_id)
        self.assertEqual(BASIC_EXAMPLE['mcic_name'], sot.mcic_name)
        self.assertEqual(BASIC_EXAMPLE['mcic_status'], sot.mcic_status)
        self.assertEqual(BASIC_EXAMPLE['tenant_id'], sot.tenant_id)
        self.assertEqual(BASIC_EXAMPLE['tenant_name'], sot.tenant_name)
        self.assertEqual(BASIC_EXAMPLE['service_type'], sot.service_type)
        self.assertEqual(BASIC_EXAMPLE['ngc'], sot.ngc)
        self.assertEqual(BASIC_EXAMPLE['colo'], sot.colo)
        self.assertEqual(BASIC_EXAMPLE['timezone'], sot.timezone)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
The :class:`~ecl.service_filter.ServiceFilter` is the base class
for service identifiers and user service preferences.  Each
:class:`~ecl.resource.Resource` has a service identifier to
associate the resource with a service.  An example of a service identifier
would be ``ecl.compute.compute_service.ComputeService``.
The preferences are stored in the
:class:`~ecl.profile.Profile` object.
The service preference and the service identifier are joined to create a
filter to match a service.

Examples
--------

The :class:`~ecl.service_filter.ServiceFilter` class can be built
with a service type, interface, region, name, and version.

Create a service filter
~~~~~~~~~~~~~~~~~~~~~~~

Create a compute service and service preference. Join the services
and match::

    from ecl import service_filter
    from ecl.compute import compute_service
    default = compute_service.ComputeService()
    preference = service_filter.ServiceFilter('compute', version='v2')
    result = preference.join(default)
    matches = (result.match_service_type('compute') and
               result.match_service_name('Hal9000') and
               result.match_region('DiscoveryOne') and
               result.match_interface('public'))
    print(str(result))
    print("matches=" + str(matches))

The resulting output from the code::

    service_type=compute,interface=public,version=v2
    matches=True
"""


class ValidVersion(object):

    def __init__(self, module, path=None):
        """" Valid service version.

        :param string module: Module associated with version.
        :param string path: URL path version.
        """
        self.module = module
        self.path = path or module


class ServiceFilter(dict):
    UNVERSIONED = ''
    PUBLIC = 'public'
    INTERNAL = 'internal'
    ADMIN = 'admin'
    valid_versions = []

    def __init__(self, service_type, interface=PUBLIC, region=None,
                 service_name=None, version=None, api_version=None,
                 requires_project_id=False):
        """Create a service identifier.

        :param string service_type: The desired type of service.
        :param string interface: The exposure of the endpoint. Should be
                                  `public` (default), `internal` or `admin`.
        :param string region: The desired region (optional).
        :param string service_name: Name of the service
        :param string version: Version of service to use.
        :param string api_version: Microversion of service supported.
        :param bool requires_project_id: True if this service's endpoint
                                         expects project id to be included.
        """
        self['service_type'] = service_type.lower()
        self['interface'] = interface
        self['region_name'] = region
        self['service_name'] = service_name
        self['version'] = version
        self['api_version'] = api_version
        self['requires_project_id'] = requires_project_id

    @property
    def service_type(self):
        return self['service_type']

    @property
    def interface(self):
        return self['interface']

    @interface.setter
    def interface(self, value):
        self['interface'] = value

    @property
    def region(self):
        return self['region_name']

    @region.setter
    def region(self, value):
        self['region_name'] = value

    @property
    def service_name(self):
        return self['service_name']

    @service_name.setter
    def service_name(self, value):
        self['service_name'] = value

    @property
    def version(self):
        return self['version']

    @version.setter
    def version(self, value):
        self['version'] = value

    @property
    def api_version(self):
        return self['api_version']

    @api_version.setter
    def api_version(self, value):
        self['api_version'] = value

    @property
    def requires_project_id(self):
        return self['requires_project_id']

    @requires_project_id.setter
    def requires_project_id(self, value):
        self['requires_project_id'] = value

    @property
    def path(self):
        return self['path']

    @path.setter
    def path(self, value):
        self['path'] = value

    def get_path(self, version=None):
        if not self.version:
            self.version = version
        return self.get('path', self._get_valid_version().path)

    def get_filter(self):
        filter = dict(self)
        del filter['version']
        return filter

    def _get_valid_version(self):
        if self.valid_versions:
            if self.version:
                for valid in self.valid_versions:
                    # NOTE(thowe): should support fuzzy match e.g: v2.1==v2
                    if self.version.startswith(valid.module):
                        return valid
            return self.valid_versions[0]
        return ValidVersion('')

    def get_module(self):
        """Get the full module name associated with the service."""
        module = self.__class__.__module__.split('.')
        module = ".".join(module[:-1])
        module = module + "." + self._get_valid_version().module
        return module

    def get_service_module(self):
        """Get the module version of the service name.

        This would often be the same as the service type except in cases like
        object store where the service type is `object-store` and the module
        is `object_store`.
        """
        return self.__class__.__module__.split('.')[1]



# -*- coding: utf-8 -*-

import base
from ecl.network import network_service
from ecl import resource2
from ecl import exceptions
from ecl import utils


class Firewall(base.NetworkBaseResource):

    resources_key = "firewalls"
    resource_key = "firewall"
    service = network_service.NetworkService("v2.0")
    base_path = '/' + service.version + "/firewalls"

    _query_mapping = resource2.QueryParameters(
        "admin_username", "availability_zone",
        "default_gateway", "description",
        "firewall_plan_id", "id",
        "name", "status", "tenant_id",
        "user_username", "sort_key", "sort_dir",
    )

    # Capabilities
    allow_list = True
    allow_create = True
    allow_get = True
    allow_update = True
    allow_delete = True

    # Properties
    #: Username with admin access to VM instance.
    admin_username = resource2.Body("admin_username")
    #: User password with admin access to VM instance.
    admin_password = resource2.Body("admin_password")
    #: Availability Zone.
    availability_zone = resource2.Body("availability_zone")
    #: IP address of default gateway.
    default_gateway = resource2.Body("default_gateway")
    #: Description of the Firewall.
    description = resource2.Body("description")
    #: Firewall Plan.
    firewall_plan_id = resource2.Body("firewall_plan_id")
    #: Unique ID of the Firewall.
    id = resource2.Body("id")
    #: Name of the Firewall.
    name = resource2.Body("name")
    #: The Firewall status.
    status = resource2.Body("status")
    #: Tenant ID of the owner (UUID).
    tenant_id = resource2.Body("tenant_id")
    #: Username with normal user access to VM instance.
    user_username = resource2.Body("user_username")
    #: User password with normal access to VM instance.
    user_password = resource2.Body("user_password")
    #: Attached interfaces:
    #: interfaces.id: Unique ID of the Firewall Interface.
    #: interfaces.ip_address: IP Address.
    #: interfaces.name: Name the Firewall Interface.
    #: interfaces.network_id: Network.
    #: interfaces.slot_number: Slot Number.
    #: interfaces.virtual_ip_address: Virtual IP Address.
    #: interfaces.virtual_ip_properties: Properties used for virtual IP address.
    #: interfaces.virtual_ip_properties.protocol: Redundancy Protocol.
    #: interfaces.virtual_ip_properties.vrid: VRRP group identifier.
    interfaces = resource2.Body("interfaces")

    def update(self, session, firewall_id, **attrs):
        """Update a firewall."""

        uri = utils.urljoin(self.base_path, firewall_id)
        resp = session.put(
            uri, endpoint_filter=self.service,
            json=attrs
        )
        self._translate_response(resp, has_body=True)
        return self

    @classmethod
    def find(cls, session, name_or_id, ignore_missing=False, **params):
        """Find a resource by its name or id.

        :param session: The session to use for making this request.
        :type session: :class:`~ecl.session.Session`
        :param name_or_id: This resource's identifier, if needed by
                           the request. The default is ``None``.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :param dict params: Any additional parameters to be passed into
                            underlying methods, such as to
                            :meth:`~ecl.resource2.Resource.existing`
                            in order to pass on URI parameters.

        :return: The :class:`Resource` object matching the given name or id
                 or None if nothing matches.
        :raises: :class:`ecl.exceptions.DuplicateResource` if more
                 than one resource is found for this request.
        :raises: :class:`ecl.exceptions.ResourceNotFound` if nothing
                 is found and ignore_missing is ``False``.
        """
        # Try to short-circuit by looking directly for a matching ID.

        data = cls.list(session, **params)

        result = cls._get_one_match(name_or_id, data)
        if result is not None:
            return result

        if ignore_missing:
            return None
        raise exceptions.ResourceNotFound(
            "No %s found for %s" % (cls.__name__, name_or_id))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import six
from ecl.tests.functional import base

class TestScheme(base.BaseFunctionalTest):
    image_id = "1b782d43-4e43-4d96-883b-ff423f8b8b7c"

    def test_01_get_image_schema(self):
        scheme = self.conn.image.get_image_schema()
        self.assertIsInstance(scheme.additionalProperties, dict)
        self.assertIsInstance(scheme.links, list)
        self.assertIsInstance(scheme.name, six.string_types)
        self.assertIsInstance(scheme.properties, dict)

    def test_02_get_images_schema(self):
        scheme = self.conn.image.get_images_schema()
        self.assertIsInstance(scheme.links, list)
        self.assertIsInstance(scheme.name, six.string_types)
        self.assertIsInstance(scheme.properties, dict)

    def test_03_get_image_member_schema(self):
        scheme = self.conn.image.get_image_member_schema()
        self.assertIsInstance(scheme.name, six.string_types)
        self.assertIsInstance(scheme.properties, dict)

    def test_04_get_metadata_namespace_schema(self):
        scheme = self.conn.image.get_metadata_namespace_schema()
        #self.assertIsInstance(scheme.additionalProperties, dict)
        self.assertIsInstance(scheme.definitions, dict)
        self.assertIsInstance(scheme.name, six.string_types)
        self.assertIsInstance(scheme.properties, dict)

    def test_05_get_metadata_object_schema(self):
        scheme = self.conn.image.get_metadata_object_schema()
        #self.assertIsInstance(scheme.additionalProperties, dict)
        self.assertIsInstance(scheme.definitions, dict)
        self.assertIsInstance(scheme.name, six.string_types)
        self.assertIsInstance(scheme.properties, dict)

    def test_06_get_metadata_property_schema(self):
        scheme = self.conn.image.get_metadata_property_schema()
        #self.assertIsInstance(scheme.additionalProperties, dict)
        self.assertIsInstance(scheme.definitions, dict)
        self.assertIsInstance(scheme.name, six.string_types)
        self.assertIsInstance(scheme.properties, dict)

    def test_07_get_metadata_resource_type_schema(self):
        scheme = self.conn.image.get_metadata_property_schema()
        # self.assertIsInstance(scheme.additionalProperties, dict)
        self.assertIsInstance(scheme.definitions, dict)
        self.assertIsInstance(scheme.name, six.string_types)
        self.assertIsInstance(scheme.properties, dict)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import six

from ecl import exceptions
from ecl.tests.functional import base


class TestFlavor(base.BaseFunctionalTest):

    @classmethod
    def setUpClass(cls):
        super(TestFlavor, cls).setUpClass()

        cls.one_flavor = list(cls.conn.baremetal.flavors())[0]

    def test_flavors(self):
        flavors = list(self.conn.baremetal.flavors())
        self.assertGreater(len(flavors), 0)

        for flavor in flavors:
            self.assertIsInstance(flavor.id, six.string_types)
            self.assertIsInstance(flavor.links, list)

    def test_flavors_detail(self):
        flavors = list(self.conn.baremetal.flavors_detail())
        self.assertGreater(len(flavors), 0)

        for flavor in flavors:
            self.assertIsInstance(flavor.id, six.string_types)
            self.assertIsInstance(flavor.name, six.string_types)
            self.assertIsInstance(flavor.links, list)
            self.assertIsInstance(flavor.ram, int)
            self.assertIsInstance(flavor.vcpus, int)
            self.assertIsInstance(flavor.disk, int)

    def test_get_flavor(self):
        flavor = self.conn.baremetal.get_flavor(self.one_flavor.id)
        self.assertEqual(flavor.id, self.one_flavor.id)

    def test_find_flavor_by_id(self):
        flavor = self.conn.baremetal.find_flavor(self.one_flavor.id)
        self.assertEqual(flavor.id, self.one_flavor.id)

    def test_find_flavor_by_name(self):
        flavor = self.conn.baremetal.find_flavor(self.one_flavor.name)
        self.assertEqual(flavor.id, self.one_flavor.id)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import six
from ecl.tests.functional import base

class TestContract(base.BaseFunctionalTest):
    def test_contracts(self):
        contracts = list(self.conn.sss.contracts("channel_id-xxxxx"))
        for i in contracts:
            print i.id + " " + i.login_id
        self.assertGreater(len(contracts), 0)

    @classmethod
    def test_01_create_contract(cls):
        contract = cls.conn.sss.create_contract(
            login_id="test001",
            mail_address="test@ntt.com",
            password="dsrtf34ht",
            external_reference_id="GXXXXXXXX",
            channel_id="chxxxxxxx",
            notify_password="true",
        )
        cls.new_contract_id = contract.id
        assert isinstance(contract.login_id, six.string_types)
        assert isinstance(contract.mail_address, six.string_types)
        assert isinstance(contract.id, six.string_types)
        assert isinstance(contract.user_id, six.string_types)
        assert isinstance(contract.keystone_name, six.string_types)
        assert isinstance(contract.keystone_password, six.string_types)
        assert isinstance(contract.keystone_endpoint, six.string_types)
        assert isinstance(contract.sss_endpoint, six.string_types)
        assert isinstance(contract.password, six.string_types)

    def test_02_delete_contract(self):
        self.conn.sss.delete_contract(self.new_contract_id)
        self.conn.sss.delete_contract("contract_xxxxx")

    def test_03_get_contract(self):
        contract = self.conn.sss.get_contract("econ1000015949")
        assert isinstance(contract.id, six.string_types)
        assert isinstance(contract.status, six.string_types)
        assert isinstance(contract.customer_number, six.string_types)
        assert isinstance(contract.channel_name, six.string_types)
        assert isinstance(contract.channel_id, six.string_types)
        assert isinstance(contract.management_channel, bool)
        assert isinstance(contract.contract_owner_user_id,
                          six.string_types)
        assert isinstance(contract.start_time, six.string_types)
        assert isinstance(contract.end_time, six.string_types)
        assert isinstance(contract.login_integration, six.string_types)
        assert isinstance(contract.sss_endpoint, six.string_types)
        assert isinstance(contract.sss_endpoint, six.string_types)
        assert isinstance(contract.company_code, six.string_types)
        assert isinstance(contract.glass_customer_id, six.string_types)
        assert isinstance(contract.glass_user_id, six.string_types)

    def test_04_get_billing_info(self):

        bill = self.conn.sss.get_billing_info("econ1000015949", "2016-02")
        assert isinstance(bill.id, six.string_types)
        assert isinstance(bill.customer_number, six.string_types)
        assert isinstance(bill.charge_status, six.string_types)
        assert isinstance(bill.cycle_start_time, six.string_types)
        assert isinstance(bill.cycle_end_time, six.string_types)
        assert isinstance(bill.charge_data, dict)

    def test_05_get_monthly_billing_of_each_contract(self):
        bill = self.conn.sss.get_monthly_billing_of_each_contract(
            "econ1000015949", "2016-02", "econ1000015949"
        )

        assert isinstance(bill.id, six.string_types)
        assert isinstance(bill.owner_contract_id, six.string_types)
        assert isinstance(bill.customer_number, six.string_types)
        assert isinstance(bill.charge_status, six.string_types)
        assert isinstance(bill.cycle_start_time, six.string_types)
        assert isinstance(bill.cycle_end_time, six.string_types)
        assert isinstance(bill.charge_data, dict)



# -*- coding: utf-8 -*-

from ecl import service_filter


class ProviderConnectivityService(service_filter.ServiceFilter):
    """The provider connectivity service."""

    valid_versions = [
        service_filter.ValidVersion('v1'),
        service_filter.ValidVersion('v2'),
    ]

    def __init__(self, version=None):
        """Create a provider connectivity service."""
        super(ProviderConnectivityService, self).__init__(
            service_type='provider-connectivity',
            version=version
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os
import os_client_config
import time
import unittest

from keystoneauth1 import exceptions as _exceptions
from ecl import connection


#: Defines the OpenStack Client Config (OCC) cloud key in your OCC config
#: file, typically in $HOME/.config/ecl/clouds.yaml. That configuration
#: will determine where the functional tests will be run and what resource
#: defaults will be used to run the functional tests.
TEST_CLOUD = os.getenv('OS_CLOUD', 'test_cloud')


class Opts(object):
    def __init__(self, cloud_name='test_cloud', debug=False):
        self.cloud = cloud_name
        self.debug = debug


def _get_resource_value(resource_key, default):
    try:
        return cloud.config['functional'][resource_key]
    except KeyError:
        return default

opts = Opts(cloud_name=TEST_CLOUD)
occ = os_client_config.OpenStackConfig()
cloud = occ.get_one_cloud(opts.cloud, argparse=opts)

IMAGE_NAME = _get_resource_value('image_name', 'cirros-0.3.4-x86_64-uec')
FLAVOR_NAME = _get_resource_value('flavor_name', '1CPU-4GB')


def service_exists(**kwargs):
    """Decorator function to check whether a service exists

    Usage:
    @unittest.skipUnless(base.service_exists(service_type="metering"),
                         "Metering service does not exist")
    class TestMeter(base.BaseFunctionalTest):
        ...

    :param kwargs: The kwargs needed to filter an endpoint.
    :returns: True if the service exists, otherwise False.
    """
    try:
        conn = connection.from_config(cloud_name=TEST_CLOUD)
        conn.session.get_endpoint(**kwargs)

        return True
    except _exceptions.EndpointNotFound:
        return False


class BaseFunctionalTest(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.conn = connection.from_config(cloud_name=TEST_CLOUD)

    @classmethod
    def assertIs(cls, expected, actual):
        if expected != actual:
            raise Exception(expected + ' != ' + actual)

    @classmethod
    def linger_for_delete(cls):
        time.sleep(40)



# -*- coding: utf-8 -*-

from ecl.network.v2.static_route import StaticRoute
from ecl.tests.functional import base


class TestStaticRoute(base.BaseFunctionalTest):

    @classmethod
    def prepare_create(cls):
        for inet in cls.conn.network.internet_gateways():
            if inet.name == "Test-sdk-inet-gw":
                cls.one_inet_gw = inet
                break

        for pip in cls.conn.network.publicips():
            cls.one_pip = pip
            break

        for gwif in cls.conn.network.gateway_interfaces():
            if gwif.name == "Test-sdk-gwi":
                cls.one_gwif = gwif

    @classmethod
    def setUpClass(cls):
        super(TestStaticRoute, cls).setUpClass()
        cls.one_sr = None
        cls.one_sr_name = None
        cls.one_inet_gw = None
        cls.one_pip = None
        cls.one_gwif = None
        srs = cls.conn.network.static_routes()
        for sr in srs:
            if sr.name == cls.one_sr_name:
                cls.one_sr = sr
                break
        if cls.one_sr is None:
            cls.prepare_create()
            cls.one_sr = cls.conn.network.create_static_route(
                "internet",
                cls.one_pip.cidr, "192.168.1.115",
                name=cls.one_sr_name,
                internet_gw_id=cls.one_inet_gw.id,
            )
            cls.one_sr.wait_for_create(cls.conn.network.session)

    def test_get(self):
        if self.one_sr:
            sot = self.conn.network.get_static_route(self.one_sr.id)
            self.assertEqual(self.one_sr.id, sot.id)

    def test_update(self):
        if self.one_sr:
            sot = self.conn.network.update_static_route(self.one_sr.id,
                                                        description="Test SDK")
            self.assertIsInstance(sot, StaticRoute)

    def test_delete(self):
        if self.one_sr:
            sot = self.conn.network.delete_static_route(self.one_sr.id)
            self.assertIsInstance(sot, StaticRoute)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import testtools

from ecl.compute.v2 import image

IDENTIFIER = 'IDENTIFIER'
BASIC_EXAMPLE = {
    'id': IDENTIFIER,
    'links': '2',
    'name': '3',
}

DETAILS = {
    'created': '2015-03-09T12:14:57.233772',
    'metadata': {'key': '2'},
    'minDisk': 3,
    'minRam': 4,
    'progress': 5,
    'status': '6',
    'updated': '2015-03-09T12:15:57.233772',
    'OS-EXT-IMG-SIZE:size': 8
}

DETAIL_EXAMPLE = BASIC_EXAMPLE.copy()
DETAIL_EXAMPLE.update(DETAILS)


class TestImage(testtools.TestCase):

    def test_basic(self):
        sot = image.Image()
        self.assertEqual('image', sot.resource_key)
        self.assertEqual('images', sot.resources_key)
        self.assertEqual('/images', sot.base_path)
        self.assertEqual('compute', sot.service.service_type)
        self.assertFalse(sot.allow_create)
        self.assertTrue(sot.allow_get)
        self.assertFalse(sot.allow_update)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)

        self.assertDictEqual({"server": "server",
                              "name": "name",
                              "status": "status",
                              "type": "type",
                              "min_disk": "minDisk",
                              "min_ram": "minRam",
                              "changes_since": "changes-since"},
                             sot._query_mapping._mapping)

    def test_make_basic(self):
        sot = image.Image(**BASIC_EXAMPLE)
        self.assertEqual(BASIC_EXAMPLE['id'], sot.id)
        self.assertEqual(BASIC_EXAMPLE['links'], sot.links)
        self.assertEqual(BASIC_EXAMPLE['name'], sot.name)

    def test_detail(self):
        sot = image.ImageDetail()
        self.assertEqual('image', sot.resource_key)
        self.assertEqual('images', sot.resources_key)
        self.assertEqual('/images/detail', sot.base_path)
        self.assertEqual('compute', sot.service.service_type)
        self.assertFalse(sot.allow_create)
        self.assertFalse(sot.allow_get)
        self.assertFalse(sot.allow_update)
        self.assertFalse(sot.allow_delete)
        self.assertTrue(sot.allow_list)

    def test_make_detail(self):
        sot = image.ImageDetail(**DETAIL_EXAMPLE)
        self.assertEqual(DETAIL_EXAMPLE['created'], sot.created_at)
        self.assertEqual(DETAIL_EXAMPLE['id'], sot.id)
        self.assertEqual(DETAIL_EXAMPLE['links'], sot.links)
        self.assertEqual(DETAIL_EXAMPLE['metadata'], sot.metadata)
        self.assertEqual(DETAIL_EXAMPLE['minDisk'], sot.min_disk)
        self.assertEqual(DETAIL_EXAMPLE['minRam'], sot.min_ram)
        self.assertEqual(DETAIL_EXAMPLE['name'], sot.name)
        self.assertEqual(DETAIL_EXAMPLE['progress'], sot.progress)
        self.assertEqual(DETAIL_EXAMPLE['status'], sot.status)
        self.assertEqual(DETAIL_EXAMPLE['updated'], sot.updated_at)
        self.assertEqual(DETAIL_EXAMPLE['OS-EXT-IMG-SIZE:size'], sot.size)



# -*- coding: utf-8 -*-

from ecl.provider_connectivity import provider_connectivity_service
from ecl import resource2


class ExchangePoint(resource2.Resource):
    resources_key = "exchange_points"
    service = provider_connectivity_service.ProviderConnectivityService("v2.0")
    base_path = '/' + service.version + '/exchange_points'

    # Capabilities
    allow_list = True

    # Properties
    #: It identifies exchange_point resource uniquely.
    id = resource2.Body('id')
    #: Pair of location name of AWS and ECL2.0.
    location_names = resource2.Body('location_names')
    #: Name of service_type
    service_type = resource2.Body('service_type')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid

from ecl.compute.v2 import keypair
from ecl.tests.functional import base


class TestKeypair(base.BaseFunctionalTest):

    @classmethod
    def setUpClass(cls):
        super(TestKeypair, cls).setUpClass()
        cls.one_keypair = None
        sots = cls.conn.compute.keypairs()
        if sots and len(sots) > 0:
            cls.one_keypair = sots[0]

    def test_list(self):
        names = [o.name for o in self.conn.compute.keypairs()]
        print names
        self.assertIn(self.one_keypair.name, names)

    def test_find(self):
        sot = self.conn.compute.find_keypair(self.one_keypair.name)
        self.assertEqual(self.one_keypair.id, sot.id)

    def test_get(self):
        sot = self.conn.compute.get_keypair(self.one_keypair.name)
        self.assertEqual(self.one_keypair.name, sot.name)
        self.assertEqual(self.one_keypair.id, sot.id)

    def test_create(self):
        test_name = "Test-Keypair"
        sot = self.conn.compute.create_keypair(test_name)
        self.assertIsInstance(sot, keypair.Keypair)

    def test_delete(self):
        sot = self.conn.compute.delete_keypair("Test-Keypair")
        self.assertIsNone(sot)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import testtools

from ecl.block_store.v2 import snapshot

FAKE_ID = "ffa9bc5e-1172-4021-acaf-cdcd78a9584d"

SNAPSHOT = {
    "status": "creating",
    "description": "Daily backup",
    "created_at": "2015-03-09T12:14:57.233772",
    "metadata": {},
    "volume_id": "5aa119a8-d25b-45a7-8d1b-88e127885635",
    "size": 1,
    "id": FAKE_ID,
    "name": "snap-001",
    "force": "true",
}

DETAILS = {
    "os-extended-snapshot-attributes:progress": "100%",
    "os-extended-snapshot-attributes:project_id":
        "0c2eba2c5af04d3f9e9d0d410b371fde"
}

DETAILED_SNAPSHOT = SNAPSHOT.copy()
DETAILED_SNAPSHOT.update(**DETAILS)


class TestSnapshot(testtools.TestCase):

    def test_basic(self):
        sot = snapshot.Snapshot(SNAPSHOT)
        self.assertEqual("snapshot", sot.resource_key)
        self.assertEqual("snapshots", sot.resources_key)
        self.assertEqual("id", sot.id_attribute)
        self.assertEqual("/snapshots", sot.base_path)
        self.assertEqual("volume", sot.service.service_type)
        self.assertTrue(sot.allow_update)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_retrieve)
        self.assertTrue(sot.allow_delete)
        self.assertFalse(sot.allow_list)

    def test_create_basic(self):
        sot = snapshot.Snapshot(SNAPSHOT)
        self.assertEqual(SNAPSHOT["id"], sot.id)
        self.assertEqual(SNAPSHOT["status"], sot.status)
        self.assertEqual(SNAPSHOT["created_at"], sot.created_at)
        self.assertEqual(SNAPSHOT["metadata"], sot.metadata)
        self.assertEqual(SNAPSHOT["volume_id"], sot.volume_id)
        self.assertEqual(SNAPSHOT["size"], sot.size)
        self.assertEqual(SNAPSHOT["name"], sot.name)
        self.assertTrue(sot.is_forced)


class TestSnapshotDetail(testtools.TestCase):

    def test_basic(self):
        sot = snapshot.SnapshotDetail(DETAILED_SNAPSHOT)
        self.assertIsInstance(sot, snapshot.Snapshot)
        self.assertEqual("/snapshots/detail", sot.base_path)

    def test_create_detailed(self):
        sot = snapshot.SnapshotDetail(DETAILED_SNAPSHOT)

        self.assertEqual(
            DETAILED_SNAPSHOT["os-extended-snapshot-attributes:progress"],
            sot.progress)
        self.assertEqual(
            DETAILED_SNAPSHOT["os-extended-snapshot-attributes:project_id"],
            sot.project_id)



# -*- coding: utf-8 -*-


import six

from ecl import exceptions
from ecl.tests.functional import base


class TestCIC(base.BaseFunctionalTest):

    @classmethod
    def setUpClass(cls):
        super(TestCIC, cls).setUpClass()
        mcics = list(cls.conn.connectivity.mcics())
        cls.one_mcic = None
        cls.one_cic = None
        if mcics and len(mcics) > 0:
            cls.one_mcic = mcics[0]
        if cls.one_mcic is None:
            return

        cics = list(cls.conn.connectivity.cics(cls.one_mcic.mcic_id))
        if cics and len(cics) > 0:
            cls.one_cic = cics[0]

    def test_cics(self):
        cics = list(self.conn.connectivity.cics(self.one_mcic.mcic_id))
        self.assertGreaterEqual(len(cics), 0)

        for cic in cics:
            self.assertIsInstance(cic.cic_id, six.string_types)
            if cic.cic_name is not None:
                self.assertIsInstance(cic.cic_name, six.string_types)
            if cic.cic_status is not None:
                self.assertIsInstance(cic.cic_status, six.string_types)
            if cic.network_id is not None:
                self.assertIsInstance(cic.network_id, six.string_types)
            if cic.network_name is not None:
                self.assertIsInstance(cic.network_name, six.string_types)
            if cic.destination_vlan is not None:
                self.assertIsInstance(cic.destination_vlan, int)
            if cic.bandwidth is not None:
                self.assertIsInstance(cic.bandwidth, int)

    def test_get_cic_by_id(self):
        cic = self.conn.connectivity.get_cic(self.one_mcic.mcic_id,
                                             self.one_cic.cic_id)
        self.assertEqual(cic.cic_id, self.one_cic.cic_id)
        self.assertEqual(cic.cic_name, self.one_cic.cic_name)
        self.assertEqual(cic.cic_status, self.one_cic.cic_status)
        self.assertEqual(cic.network_id, self.one_cic.network_id)
        self.assertEqual(cic.network_name, self.one_cic.network_name)
        self.assertEqual(cic.destination_vlan, self.one_cic.destination_vlan)
        self.assertEqual(cic.bandwidth, self.one_cic.bandwidth)

    def test_create_cic(self):
        cic_name = self.one_mcic.mcic_name + "-xxx"
        network_id = self.one_cic.network_id
        colo_vlan = 4
        cic = self.conn.connectivity.create_cic(self.one_mcic.mcic_id,
                                                cic_name,
                                                network_id,
                                                colo_vlan)
        self.assertIsInstance(cic.cic_id, six.string_types)

    def test_delete_cic(self):
        cic_id = self.one_cic.cic_id
        mcic_id = self.one_mcic.mcic_id
        cic = self.conn.connectivity.delete_cic(mcic_id, cic_id)
        # print cic.to_dict()
        self.assertEqual(cic_id, cic.id)

    def test_update_cic(self):
        cic_id = self.one_cic.cic_id
        mcic_id = self.one_mcic.mcic_id
        cic = self.conn.connectivity.update_cic(mcic_id, cic_id, "xxx")
        # print cic.to_dict()
        self.assertEqual(cic_id, cic.id)



# Licensed under the Apache License, Version 2.0 (the "License"); you may

# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl.object_store.v1 import _base
from ecl import resource


class Container(_base.BaseResource):
    _custom_metadata_prefix = "X-Container-Meta-"
    _system_metadata = {
        "content_type": "content-type",
        "is_content_type_detected": "x-detect-content-type",
        "versions_location": "x-versions-location",
        "read_acl": "x-container-read",
        "write_acl": "x-container-write",
        "sync_to": "x-container-sync-to",
        "sync_key": "x-container-sync-key",
        "meta_name": "X-Container-Meta-name",
        "meta_access_control_allow_origin":
            "X-Container-Meta-Access-Control-Allow-Origin",
        "meta_access_control_max_age":
            "X-Container-Meta-Access-Control-Max-Age",
        "meta_access_control_expose_headers":
            "X-Container-Meta-Access-Control-Expose-Headers",
        "meta_quota_bytes": "X-Container-Meta-Quota-Bytes",
        "meta_quota_count": "X-Container-Meta-Quota-Count",
        "meta_temp_url_key": "X-Container-Meta-Temp-URL-Key",
        "meta_temp_url_key_2": "X-Container-Meta-Temp-URL-Key-2",
    }

    base_path = "/"
    id_attribute = "name"

    allow_create = True
    allow_retrieve = True
    allow_update = True
    allow_delete = True
    allow_list = True
    allow_head = True

    # Container body data (when id=None)
    #: The name of the container.
    name = resource.prop("name")
    #: The number of objects in the container.
    count = resource.prop("count")
    #: The total number of bytes that are stored in Object Storage
    #: for the container.
    bytes = resource.prop("bytes")

    # Container metadata (when id=name)
    #: The number of objects.
    object_count = resource.header("x-container-object-count", type=int)
    #: The count of bytes used in total.
    bytes_used = resource.header("x-container-bytes-used", type=int)
    #: The timestamp of the transaction. (date created)
    timestamp = resource.header("x-timestamp")

    # Request headers (when id=None)
    #: If set to True, Object Storage queries all replicas to return the
    #: most recent one. If you omit this header, Object Storage responds
    #: faster after it finds one valid replica. Because setting this
    #: header to True is more expensive for the back end, use it only
    #: when it is absolutely needed. *Type: bool*
    is_newest = resource.header("x-newest", type=bool)

    # Request headers (when id=name)
    #: The ACL that grants read access. If not set, this header is not
    #: returned by this operation.
    read_acl = resource.header("x-container-read")
    #: The ACL that grants write access. If not set, this header is not
    #: returned by this operation.
    write_acl = resource.header("x-container-write")
    #: The destination for container synchronization. If not set,
    #: this header is not returned by this operation.
    sync_to = resource.header("x-container-sync-to")
    #: The secret key for container synchronization. If not set,
    #: this header is not returned by this operation.
    sync_key = resource.header("x-container-sync-key")
    #: Enables versioning on this container. The value is the name
    #: of another container. You must UTF-8-encode and then URL-encode
    #: the name before you include it in the header. To disable
    #: versioning, set the header to an empty string.
    versions_location = resource.header("x-versions-location")
    #: The MIME type of the list of names.
    content_type = resource.header("content-type")
    #: If set to true, Object Storage guesses the content type based
    #: on the file extension and ignores the value sent in the
    #: Content-Type header, if present. *Type: bool*
    is_content_type_detected = resource.header("x-detect-content-type",
                                               type=bool)
    #: In combination with Expect: 100-Continue, specify an
    #: "If-None-Match: \*" header to query whether the server already
    #: has a copy of the object before any data is sent.
    if_none_match = resource.header("if-none-match")
    #: The container metadata, where name is the name of metadata item.
    #: You must specify an X-Container-Meta-name header for each metadata item
    #: (for each name ) that you want to add or update.
    meta_name = resource.header("X-Container-Meta-name")
    #: Originating URLs allowed to make cross-origin requests (CORS),
    #: separated by spaces.
    meta_access_control_allow_origin = resource.header(
        "X-Container-Meta-Access-Control-Allow-Origin")
    #: Maximum time for the origin to hold the preflight results.
    meta_access_control_max_age = resource.header(
        "X-Container-Meta-Access-Control-Max-Age")
    #: Headers the Object Storage service exposes to the browser (technically,
    #: through the user-agent setting),
    #: in the request response, separated by spaces.
    #: By default the Object Storage service returns the following headers
    meta_access_control_expose_headers = resource.header(
        "X-Container-Meta-Access-Control-Expose-Headers")
    #: Sets maximum size of the container, in bytes.
    #: Typically these values are set by an administrator.
    meta_quota_bytes = resource.header("X-Container-Meta-Quota-Bytes")
    #: Sets maximum object count of the container.
    #: Typically these values are set by an administrator.
    meta_quota_count = resource.header("X-Container-Meta-Quota-Count")
    #: The secret key value for temporary URLs.
    meta_temp_url_key = resource.header("X-Container-Meta-Temp-URL-Key")
    #: A second secret key value for temporary URLs.
    #: The second key enables you to rotate keys by having two active keys
    #: at the same time.
    meta_temp_url_key_2 = resource.header("X-Container-Meta-Temp-URL-Key-2")

    @classmethod
    def create_by_id(cls, session, attrs, resource_id=None):
        """Create a Resource from its attributes.

        :param session: The session to use for making this request.
        :type session: :class:`~ecl.session.Session`
        :param dict attrs: The attributes to be sent in the body
                           of the request.
        :param resource_id: This resource's identifier, if needed by
                            the request. The default is ``None``.

        :return: A ``dict`` representing the response headers.
        """
        url = cls._get_url(None, resource_id)
        headers = attrs.get(resource.HEADERS, dict())
        headers['Accept'] = ''
        return session.put(url, endpoint_filter=cls.service,
                           headers=headers).headers

    def create(self, session):
        """Create a Resource from this instance.

        :param session: The session to use for making this request.
        :type session: :class:`~ecl.session.Session`

        :return: This instance.
        """
        resp = self.create_by_id(session, self._attrs, self.id)
        self.set_headers(resp)
        self._reset_dirty()
        return self



# -*- coding: utf-8 -*-

from ecl.provider_connectivity import provider_connectivity_service
from ecl.provider_connectivity.v2.base import ProviderConnectivityBaseResource
from ecl import resource2


class GCPConnection(ProviderConnectivityBaseResource):
    resources_key = "gcp_connections"
    resource_key = "gcp_connection"
    service = provider_connectivity_service.ProviderConnectivityService("v2.0")
    base_path = '/' + service.version + '/gcp_connections'

    # Capabilities
    allow_list = True
    allow_get = True
    allow_create = True
    allow_update = True
    allow_delete = True

    # _query_mapping = resource2.QueryParameters()
    # TBD

    # Properties
    #: It identifies connection resource uniquely.
    id = resource2.Body('id')
    #: Name of connection.
    name = resource2.Body('name')
    #: Description of connection.
    description = resource2.Body('description')
    #: Status of connection.
    status = resource2.Body('status')
    #: Status of operation for gcp_connection.
    operation_status = resource2.Body('operation_status')
    #: Pairing Key for Active Cloud Router in GCP.
    pairing_key_primary = resource2.Body('pairing_key_primary')
    #: Pairing Key for Standby Cloud Router in GCP.
    pairing_key_secondary = resource2.Body('pairing_key_secondary')
    #: Tenant ID of the owner.
    tenant_id = resource2.Body('tenant_id')
    #: Bandwidth assigned with this connection.
    bandwidth = resource2.Body('bandwidth')
    #: QoS type assigned with this connection.
    qos_type = resource2.Body('qos_type')
    #: ID of exchange_points.
    exchange_point_id = resource2.Body('exchange_point_id')
    #: VLAN ID.
    vlan = resource2.Body('vlan')
    #: ID of gcp_gateway which gcp_connection connected.
    gcp_gateway_id = resource2.Body('gcp_gateway_id')
    #: ID of gcp_interface which gcp_connection connected.
    gcp_interface_id = resource2.Body('gcp_interface_id')
    #: Name of Region where customer's cloud router exist.
    gcp_region_name = resource2.Body('gcp_region_name')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import mock
import testtools

from ecl.telemetry.v2 import capability

EXAMPLE = {
    "id": "123",
    "enabled": False,
}
BODY = {
    "api": {
        "statistics:query:complex": False,
        "alarms:history:query:simple": True,
        "events:query:simple": True,
        "alarms:query:simple": True,
        "resources:query:simple": True,
    }
}


class TestCapability(testtools.TestCase):
    def test_basic(self):
        sot = capability.Capability()
        self.assertEqual('capability', sot.resource_key)
        self.assertEqual('capabilities', sot.resources_key)
        self.assertEqual('/capabilities', sot.base_path)
        self.assertEqual('metering', sot.service.service_type)
        self.assertFalse(sot.allow_create)
        self.assertFalse(sot.allow_retrieve)
        self.assertFalse(sot.allow_update)
        self.assertFalse(sot.allow_delete)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = capability.Capability(EXAMPLE)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['enabled'], sot.is_enabled)

    def test_list(self):
        sess = mock.Mock()
        resp = mock.Mock()
        resp.json = mock.Mock(return_value=BODY)
        sess.get = mock.Mock(return_value=resp)

        caps = capability.Capability.list(sess)

        caps = sorted(caps, key=lambda cap: cap.id)
        self.assertEqual(5, len(caps))
        self.assertEqual('alarms:history:query:simple', caps[0].id)
        self.assertTrue(caps[0].is_enabled)
        self.assertEqual('alarms:query:simple', caps[1].id)
        self.assertTrue(caps[1].is_enabled)
        self.assertEqual('events:query:simple', caps[2].id)
        self.assertTrue(caps[2].is_enabled)
        self.assertEqual('resources:query:simple', caps[3].id)
        self.assertTrue(caps[3].is_enabled)
        self.assertEqual('statistics:query:complex', caps[4].id)
        self.assertFalse(caps[4].is_enabled)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl.image import image_service
from ecl import resource2


class Member(resource2.Resource):
    resources_key = 'members'
    service = image_service.ImageService()
    base_path = '/' + service.version + '/images/%(image_id)s/members'

    # Capabilities
    allow_create = True
    allow_get = True
    allow_update = True
    allow_delete = True
    allow_list = True

    # Properties

    # See https://bugs.launchpad.net/glance/+bug/1526991 for member/member_id
    # 'member' is documented incorrectly as being deprecated but it's the
    # only thing that works. 'member_id' is not accepted.

    #: The ID of the image member. An image member is a tenant
    #: with whom the image is shared.
    member_id = resource2.Body('member_id')
    #: The date and time when the member was created.
    created_at = resource2.Body('created_at')
    #: Image ID stored through the image API. Typically a UUID.
    image_id = resource2.URI('image_id')
    #: The status of the image.
    status = resource2.Body('status')
    #: The URL for schema of the member.
    schema = resource2.Body('schema')
    #: The date and time when the member was updated.
    updated_at = resource2.Body('updated_at')
    #: Member_id used to create a member
    member = resource2.Body('member', alternate_id=True)

    def update(self, session, image_id, member_id, status):
        """Update the member of an image."""

        uri = self.base_path + '/%(member_id)s'
        uri = uri % {"image_id":image_id, "member_id":member_id}
        resp = session.put(
            uri,
            json={"status":status},
            headers={"Accept": "application/json"},
            endpoint_filter=self.service
        )
        self._translate_response(resp, has_body=True)
        return self



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import testtools

from ecl.compute.v2 import keypair

EXAMPLE = {
    'fingerprint': '1',
    'name': '2',
    'public_key': '3',
    'private_key': '3',
}


class TestKeypair(testtools.TestCase):

    def test_basic(self):
        sot = keypair.Keypair()
        self.assertEqual('keypair', sot.resource_key)
        self.assertEqual('keypairs', sot.resources_key)
        self.assertEqual('/os-keypairs', sot.base_path)
        self.assertEqual('compute', sot.service.service_type)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_get)
        self.assertFalse(sot.allow_update)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = keypair.Keypair(**EXAMPLE)
        self.assertEqual(EXAMPLE['fingerprint'], sot.fingerprint)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['public_key'], sot.public_key)
        self.assertEqual(EXAMPLE['private_key'], sot.private_key)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import testtools

from ecl.network.v2 import network

IDENTIFIER = 'IDENTIFIER'
EXAMPLE = {
    "admin_state_up": True,
    "description": "Example network 1 description.",
    "id": IDENTIFIER,
    "name": "Example network 1",
    "plane": "data",
    "shared": True,
    "status": "ACTIVE",
    "subnets": [
      IDENTIFIER,
      IDENTIFIER
    ],
    "tags": {
      "keyword1": "value1",
      "keyword2": "value2"
    },
    "tenant_id": IDENTIFIER
}


class TestNetwork(testtools.TestCase):

    def test_basic(self):
        sot = network.Network()
        self.assertEqual('network', sot.resource_key)
        self.assertEqual('networks', sot.resources_key)
        self.assertEqual('/networks', sot.base_path)
        self.assertEqual('network', sot.service.service_type)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_get)
        self.assertTrue(sot.allow_update)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = network.Network(**EXAMPLE)
        self.assertTrue(sot.admin_state_up)
        self.assertTrue(sot.shared)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['tenant_id'], sot.project_id)
        self.assertEqual(EXAMPLE['status'], sot.status)
        self.assertEqual(EXAMPLE['subnets'], sot.subnet_ids)
        self.assertEqual(EXAMPLE['description'], sot.description)
        self.assertEqual(EXAMPLE['tags'], sot.tags)
        self.assertEqual(EXAMPLE['plane'], sot.plane)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl.baremetal import baremetal_service
from ecl import resource2


class Stock(resource2.Resource):
    resource_key = 'stock'
    base_path = '/stock'
    service = baremetal_service.BaremetalService()

    # Capabilities
    allow_get = True

    # Properties
    #: Flavor UUID of Baremetal server stock search condition.
    flavor_id = resource2.Body('flavor_id', alternate_id=True)
    #: Availability Zone name of Baremetal server stock search condition.
    availability_zone = resource2.Body('availability_zone')
    #: Baremetal server stock is available or not.
    stock = resource2.Body('stock')

    def get(self, session, flavor_id, availability_zone=None):
        uri = self.base_path + "?flavor_id=%s" % flavor_id
        if availability_zone:
            uri += "&availability_zone=%s" % availability_zone
        resp = session.get(uri, endpoint_filter=self.service,
                           headers={"Accept": "application/json"})
        self._translate_response(resp, has_body=True)
        return self



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import testtools

from ecl.telemetry.v2 import resource

IDENTIFIER = 'IDENTIFIER'
LINKS = [{'href': 'first_uri', 'rel': 'label 1', },
         {'href': 'other_uri', 'rel': 'label', }, ]
EXAMPLE = {
    'resource_id': IDENTIFIER,
    'first_sample_timestamp': '2015-03-09T12:15:57.233772',
    'last_sample_timestamp': '2015-03-09T12:15:57.233772',
    'links': LINKS,
    'metadata': {'name_one': '1', 'name_two': '2', },
    'project_id': '123',
    'source': 'abc',
    'user_id': '789'
}


class TestResource(testtools.TestCase):

    def test_basic(self):
        sot = resource.Resource(EXAMPLE)
        self.assertIsNone(sot.resource_key)
        self.assertIsNone(sot.resources_key)
        self.assertEqual('/resources', sot.base_path)
        self.assertEqual('metering', sot.service.service_type)
        self.assertFalse(sot.allow_create)
        self.assertTrue(sot.allow_retrieve)
        self.assertFalse(sot.allow_update)
        self.assertFalse(sot.allow_delete)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = resource.Resource(EXAMPLE)
        self.assertEqual(EXAMPLE['resource_id'], sot.id)
        self.assertEqual(EXAMPLE['first_sample_timestamp'],
                         sot.first_sample_at)
        self.assertEqual(EXAMPLE['last_sample_timestamp'],
                         sot.last_sample_at)
        self.assertEqual(EXAMPLE['links'], sot.links)
        self.assertEqual(EXAMPLE['metadata'], sot.metadata)
        self.assertEqual(EXAMPLE['project_id'], sot.project_id)
        self.assertEqual(EXAMPLE['resource_id'], sot.resource_id)
        self.assertEqual(EXAMPLE['source'], sot.source)
        self.assertEqual(EXAMPLE['user_id'], sot.user_id)



# -*- coding: utf-8 -*-

from ecl.tests.functional import base


class TestVPNService(base.BaseFunctionalTest):

    def test_list_vpns(self):
        vpn_services = self.conn.network.vpn_services()
        if not vpn_services or len(vpn_services) <= 0:
            return
        vpn_service = vpn_services[0]

        vpn_gateways = self.conn.network.vpn_gateways(vpn_service_id=vpn_service.id)
        if not vpn_gateways or len(vpn_gateways) <= 0:
            return
        vpn_gw = vpn_gateways[0]
        self.assertEqual(vpn_gw.vpn_service_id, vpn_service.id)

        vpn_ifs = self.conn.network.vpn_interfaces(vpn_gw_id=vpn_gw.id)
        if not vpn_ifs or len(vpn_ifs) <= 0:
            return
        print "vpn_ifs:", len(vpn_ifs), "example id:"
        vpn_if = vpn_ifs[0]
        self.assertEqual(vpn_if.vpn_gw_id, vpn_gw.id)

    def test_list_gw_interfaces(self):
        vpn_gw_ifs = self.conn.network.gateway_interfaces(service_type="vpn")
        inet_gw_ifs = self.conn.network.gateway_interfaces(service_type="internet")
        idc_gw_ifs = self.conn.network.gateway_interfaces(service_type="interdc")
        self.assertGreaterEqual(vpn_gw_ifs, 0)
        self.assertGreaterEqual(inet_gw_ifs, 0)
        self.assertGreaterEqual(idc_gw_ifs, 0)



# -*- coding: utf-8 -*-

from ecl.security_portal import security_portal_service
from ecl import resource2
from ecl import exceptions
from ecl import utils


class SecurityDeviceInterface(resource2.Resource):
    resource_key = "device_interface"
    resources_key = "device_interfaces"
    base_path = '/ecl-api/devices/%(server_id)s/interfaces'
    service = security_portal_service.SecurityPortalService()

    _query_mapping = resource2.QueryParameters(
        "tenantid",
        "usertoken",
    )

    # Capabilities
    allow_get = True
    allow_list = True

    # Properties
    #: Port id on Openstack.
    os_port_id = resource2.Body('os_port_id', alternate_id=True)
    #: Port IP address (if available).
    os_ip_address = resource2.Body('os_ip_address')
    #: Port id on the Network-based Security devices (registered in MSA).
    msa_port_id = resource2.Body('msa_port_id')
    #: Port name on Openstack.
    os_port_name = resource2.Body('os_port_name')
    #: Network Id to which Port is associated on Openstack.
    os_network_id = resource2.Body('os_network_id')
    #: Port Status on Openstack.
    os_port_status = resource2.Body('os_port_status')
    #: Port MAC address on Openstack.
    os_mac_address = resource2.Body('os_mac_address')
    #: Subnet Id to which Port is associated on Openstack.
    os_subnet_id = resource2.Body('os_subnet_id')
    #: Server id of Network-based Security devices.
    os_server_id = resource2.Body('os_server_id')

    def get(self, session, port_id):
        uri = '/ecl-api/devices/interface/%s?tenantid=%s&usertoken=%s' \
              % (port_id, session.get_project_id(), session.get_token())
        resp = session.get(uri, endpoint_filter=self.service)
        self._translate_response(resp, has_body=True)
        return self



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl.database import database_service
from ecl import resource2
from ecl import exceptions


class Database(resource2.Resource):
    resource_key = "database"
    resources_key = "databases"
    base_path = '/instances/%(instance_id)s/databases'
    service = database_service.DatabaseService()

    # Capabilities
    allow_create = True
    allow_delete = True
    allow_list = True

    allow_update = False
    allow_get = False

    # Properties
    #: Name of Database.
    name = resource2.Body('name', alternate_id=True)
    #: Character set of Database.
    character_set = resource2.Body('character_set')
    #: Collate of Database.
    collate = resource2.Body('collate')
    #: ID of instance associated with this database
    instance_id = resource2.URI('instance_id')

    def create(self, session, instance_id, **attrs):
        base = self.base_path % {"instance_id": instance_id}
        body = {"databases": [attrs]}
        resp = session.post(base, endpoint_filter=self.service, json=body,
                            headers={"Accept": "application/json"})
        self._translate_response(resp, has_body=False)
        return self



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl.sss import  sss_service
from ecl import resource2

class Channel(resource2.Resource):
    resource_key = None
    resources_key = "channels"
    base_path = "/channels?get_contracts=%(get_contracts)s"
    service = sss_service.SssService()

    #Capabilities
    allow_list = True

    #_query_mapping = resource2.QueryParameters("get_contracts")

    #Properties
    #: Channel ID.
    channel_id = resource2.Body("channel_id", alternate_id=True)
    #: Channel name.
    channel_name = resource2.Body("channel_name")
    #: This item setting will relate to language setting of E-mail that will
    #: send by ECL2.0. Please note that NOT related to user's display language
    #: setting(That setting is rely on the user browser's language setting.)
    language = resource2.Body("language")
    #: Whether this channel_id is the partner management channel(true), or
    #: not(false).
    management_channel = resource2.Body("management_channel")
    #: The parent channel ID means partner's channel ID that belongs the end
    #: user.
    parent_channel_id = resource2.Body("parent_channel_id")
    #: List of contracts has two parameters:
    #: contract_id: Contract ID.
    #: status: The status of the contract(enable or deleted).
    contracts = resource2.Body("contracts")

    def __init__(self, synchronized=False, **attrs):
        super(Channel, self).__init__(synchronized=synchronized, **attrs)
        if "get_contracts" in attrs:
            attrs.pop("get_contracts")



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl.compute import compute_service
from ecl import resource2


class Keypair(resource2.Resource):
    resource_key = 'keypair'
    resources_key = 'keypairs'
    base_path = '/os-keypairs'
    service = compute_service.ComputeService()

    # capabilities
    allow_create = True
    allow_get = True
    allow_delete = True
    allow_list = True

    # Properties
    #: The short fingerprint associated with the ``public_key`` for
    #: this keypair.
    fingerprint = resource2.Body('fingerprint')
    # NOTE: There is in fact an 'id' field. However, it's not useful
    # because all operations use the 'name' as an identifier.
    # Additionally, the 'id' field only appears *after* creation,
    # so suddenly you have an 'id' field filled in after the fact,
    # and it just gets in the way. We need to cover this up by having
    # the name be both our id and name.
    #: The id identifying the keypair
    id = resource2.Body('name')
    #: A name identifying the keypair
    name = resource2.Body('name', alternate_id=True)
    #: The private key for the keypair
    private_key = resource2.Body('private_key')
    #: The SSH public key that is paired with the server.
    public_key = resource2.Body('public_key')
    user_id = resource2.Body('user_id')

    @classmethod
    def list(cls, session, paginated=False):
        resp = session.get(cls.base_path, endpoint_filter=cls.service,
                           headers={"Accept": "application/json"})
        resp = resp.json()
        resp = resp[cls.resources_key]

        for data in resp:
            value = cls.existing(**data[cls.resource_key])
            yield value



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl.dns import dns_service
from ecl import resource2
from ecl import exceptions


class NameServer(resource2.Resource):
    resource_key = None
    resources_key = "nameservers"
    base_path = '/v2/zones/%(zone_id)s/nameservers'
    service = dns_service.DnsService()

    # Capabilities
    allow_list = True

    # Properties
    #: ID for the zone.
    zone_id = resource2.URI('zone_id')
    #: The hostname of the nameserver that the zone should be delegated to.
    hostname = resource2.Body('hostname', alternate_id=True)
    #: The priority of the nameserver. This is used to determine the order of the the nameserver listings,
    #: and which server is used in the SOA record for the zone. This parameter is not currently supported.
    priority = resource2.Body('priority')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import mock
import testtools

from ecl.compute.v2 import volume

VOLUME_RESP_BODY = {
    "displayName": "test-name",
    "displayDescription": "test-descr",
    "availabilityZone": "zone1_groupa",
    "status": "creating",
    "attachments": [{}],
    "volumeType": "nfsdriver",
    "snapshotId": None,
    "size": 15,
    "id": "e15c8756-a9d2-49b1-9192-ce898196e985",
    "createdAt": "2016-10-28T03:03:33.000000",
    "metadata": {}
}



class TestVolume(testtools.TestCase):

    def test_basic(self):
        sot = volume.Volume()
        self.assertEqual("volume", sot.resource_key)
        self.assertEqual("volumes", sot.resources_key)
        self.assertEqual("/os-volumes", sot.base_path)
        self.assertEqual("compute", sot.service.service_type)
        self.assertTrue(sot.allow_get)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_update)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = volume.Volume(**VOLUME_RESP_BODY)

        self.assertEqual(VOLUME_RESP_BODY["displayName"], sot.name)
        self.assertEqual(VOLUME_RESP_BODY["displayDescription"], sot.description)
        self.assertEqual(VOLUME_RESP_BODY["availabilityZone"], sot.availability_zone)
        self.assertEqual(VOLUME_RESP_BODY["status"], sot.status)
        self.assertEqual(VOLUME_RESP_BODY["attachments"], sot.attachments)
        self.assertEqual(VOLUME_RESP_BODY["volumeType"], sot.volume_type)
        self.assertEqual(VOLUME_RESP_BODY["snapshotId"], sot.snapshot_id)
        self.assertEqual(VOLUME_RESP_BODY["size"], sot.size)
        self.assertEqual(VOLUME_RESP_BODY["id"], sot.id)
        self.assertEqual(VOLUME_RESP_BODY["createdAt"], sot.created_at)
        self.assertEqual(VOLUME_RESP_BODY["metadata"], sot.metadata)





# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl.sss import sss_service
from ecl import resource2
from ecl import exceptions
from ecl import utils

class User(resource2.Resource):
    resource_key = None
    resources_key = 'users'
    base_path = '/users'
    service = sss_service.SssService()

    # Capabilities
    allow_create = True
    allow_get = True
    allow_delete = True
    allow_list = True
    allow_update = True
    put_create = False

    # Properties
    #: login id of the user. When this contract is tied with icp, this
    #: parameter is fixed {email}_{user_id}.
    login_id = resource2.Body('login_id')
    #: Mail address of the user.
    mail_address = resource2.Body('mail_address')
    #: User id. format is ecid[0-9]{10}.
    user_id = resource2.Body('user_id', alternate_id=True)
    #: If this user is the Super user in this contract, 1. If not, 0.
    contract_owner = resource2.Body('contract_owner')
    #: Contract ID which this user belongs. contract id format is econ[0-9]{10}.
    contract_id = resource2.Body('contract_id')
    #: This user's API key for keystone authentication.
    keystone_name = resource2.Body('keystone_name')
    #: Keystone address this user can use to get token for SSS API request.
    keystone_endpoint = resource2.Body('keystone_endpoint')
    #: This user's API secret for keystone authentication.
    keystone_password = resource2.Body('keystone_password')
    #: SSS endpoint recommended for this user
    sss_endpoint = resource2.Body('sss_endpoint')
    #: Password of the user
    password = resource2.Body('password')
    #: If this flag is set 'true', notification eamil will be sent to new
    #: user's email.
    notify_password = resource2.Body('notify_password')
    #: New password of the user
    new_password = resource2.Body('new_password')
    #: User list.
    users = resource2.Body('users')
    #: If this user's contract is tied with NTT Communications business portal,
    #: 'icp' is shown.
    login_integration = resource2.Body('login_integration')
    #: External system oriented contract id. If this user's contract is NTT
    #: Communications, customer number with 15 numbers will be shown.
    external_reference_id = resource2.Body('external_reference_id')

    def update(self, session, user_id, **attrs):
        """Update user's information"""

        uri = utils.urljoin(self.base_path, user_id)
        resp = session.put(
            uri, endpoint_filter=self.service,
            json=attrs
        )
        self._translate_response(resp, has_body=False)
        return self

    @classmethod
    def find(cls, session, name_or_id, ignore_missing=False, **params):
        """Find a resource by its name or id.

        :param session: The session to use for making this request.
        :type session: :class:`~ecl.session.Session`
        :param name_or_id: This resource's identifier, if needed by
                           the request. The default is ``None``.
        :param bool ignore_missing: When set to ``False``
                    :class:`~ecl.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :param dict params: Any additional parameters to be passed into
                            underlying methods, such as to
                            :meth:`~ecl.resource2.Resource.existing`
                            in order to pass on URI parameters.

        :return: The :class:`Resource` object matching the given name or id
                 or None if nothing matches.
        :raises: :class:`ecl.exceptions.DuplicateResource` if more
                 than one resource is found for this request.
        :raises: :class:`ecl.exceptions.ResourceNotFound` if nothing
                 is found and ignore_missing is ``False``.
        """
        # Try to short-circuit by looking directly for a matching ID.

        data = cls.list(session, **params)

        result = cls._get_one_match(name_or_id, data)
        if result is not None:
            return result

        if ignore_missing:
            return None
        raise exceptions.ResourceNotFound(
            "No %s found for %s" % (cls.__name__, name_or_id))



# -*- coding: utf-8 -*-

from ecl.tests.functional import base
from ecl.network.v2.gw_interface import GatewayInterface


class TestGWIF(base.BaseFunctionalTest):

    @classmethod
    def setUpClass(cls):
        super(TestGWIF, cls).setUpClass()
        cls.one_gwif = None
        cls.one_nw = None
        cls.one_inet_gw = None
        cls.gw_name = "Test-sdk-inet-gw"
        cls.gwif_name = "Test-sdk-gwi"
        cls.nw_name = "nttcom01"
        cls.retrieve_gw()

        gwifs = cls.conn.network.gateway_interfaces()
        for gwi in gwifs:
            if gwi.name == cls.gwif_name:
                cls.one_gwif = gwi
                break
        if cls.one_gwif is None:
            cls.one_gwif = cls.conn.network.create_gateway_interface(
                "internet", cls.one_nw.id, 24,
                "192.168.1.112", "192.168.1.113", "192.168.1.114",
                1, name=cls.gwif_name, internet_gw_id = cls.one_inet_gw.id
            )
            cls.one_gwif.wait_for_create(cls.conn.network.session)

    @classmethod
    def retrieve_gw(cls):
        inet_gws = cls.conn.network.internet_gateways()
        for inet_gw in inet_gws:
            if inet_gw.name == cls.gw_name:
                cls.one_inet_gw = inet_gw
                break

        networks = cls.conn.network.networks()
        for nw in networks:
            if nw.name == cls.nw_name:
                cls.one_nw = nw
                break

    def _test_get(self):
        if self.one_gwif:
            sot = self.conn.network.get_gateway_interface(self.one_gwif.id)
            self.assertEqual(self.one_gwif.id, sot.id)

    def test_update(self):
        if self.one_gwif:
            sot = self.conn.network.update_gateway_interface(self.one_gwif.id,
                                                             description="Test SDK")
            self.assertIsInstance(sot, GatewayInterface)

    def _test_delete(self):
        sot = self.conn.network.delete_gateway_interface(self.one_gwif.id)
        self.assertIsInstance(sot, GatewayInterface)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import testtools

from ecl.compute.v2 import flavor

IDENTIFIER = 'IDENTIFIER'
BASIC_EXAMPLE = {
    'id': IDENTIFIER,
    'links': '2',
    'name': '3',
    'disk': 4,
    'os-flavor-access:is_public': True,
    'ram': 6,
    'vcpus': 7,
    'swap': 8,
    'OS-FLV-EXT-DATA:ephemeral': 9,
    'OS-FLV-DISABLED:disabled': False,
    'rxtx_factor': 11.0
}


class TestFlavor(testtools.TestCase):

    def test_basic(self):
        sot = flavor.Flavor()
        self.assertEqual('flavor', sot.resource_key)
        self.assertEqual('flavors', sot.resources_key)
        self.assertEqual('/flavors', sot.base_path)
        self.assertEqual('compute', sot.service.service_type)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_get)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)
        self.assertFalse(sot.allow_update)

        self.assertDictEqual({"sort_key": "sort_key",
                              "sort_dir": "sort_dir",
                              "min_disk": "minDisk",
                              "min_ram": "minRam"},
                             sot._query_mapping._mapping)

    def test_make_basic(self):
        sot = flavor.Flavor(**BASIC_EXAMPLE)
        self.assertEqual(BASIC_EXAMPLE['id'], sot.id)
        self.assertEqual(BASIC_EXAMPLE['links'], sot.links)
        self.assertEqual(BASIC_EXAMPLE['name'], sot.name)
        self.assertEqual(BASIC_EXAMPLE['disk'], sot.disk)
        self.assertEqual(BASIC_EXAMPLE['os-flavor-access:is_public'],
                         sot.is_public)
        self.assertEqual(BASIC_EXAMPLE['ram'], sot.ram)
        self.assertEqual(BASIC_EXAMPLE['vcpus'], sot.vcpus)
        self.assertEqual(BASIC_EXAMPLE['swap'], sot.swap)
        self.assertEqual(BASIC_EXAMPLE['OS-FLV-EXT-DATA:ephemeral'],
                         sot.ephemeral)
        self.assertEqual(BASIC_EXAMPLE['OS-FLV-DISABLED:disabled'],
                         sot.is_disabled)
        self.assertEqual(BASIC_EXAMPLE['rxtx_factor'], sot.rxtx_factor)

    def test_detail(self):
        sot = flavor.FlavorDetail()
        self.assertEqual('flavor', sot.resource_key)
        self.assertEqual('flavors', sot.resources_key)
        self.assertEqual('/flavors/detail', sot.base_path)
        self.assertEqual('compute', sot.service.service_type)
        self.assertFalse(sot.allow_create)
        self.assertFalse(sot.allow_get)
        self.assertFalse(sot.allow_update)
        self.assertFalse(sot.allow_delete)
        self.assertTrue(sot.allow_list)



# -*- coding: utf-8 -*-

from ecl.security_order import security_order_service
from ecl import resource2
from ecl import exceptions
from ecl import utils

class HostBasedSecurity(resource2.Resource):
    resource_key = None
    resources_key = None
    base_path = '/API/SoEntryHBS'
    service = security_order_service.SecurityOrderService()

    # Capabilities
    allow_create = True
    allow_get = True
    allow_delete = True
    allow_list = True
    allow_update = True

    #: Tenant ID of the owner (UUID).
    tenant_id = resource2.Body('tenant_id')
    #: N: Order New Host-based Security.
    #: M1: Change menu of Host-based Security.
    #: M2: Change quantity of Host-based Security.
    #: C: Cancel Host-based Security.
    sokind = resource2.Body('sokind')
    #: Requested menu. Set "Managed Anti-Virus", "Managed Virtual Patch"
    #: or "Managed Host-based Security Package" to this field.
    service_order_service = resource2.Body('service_order_service')
    #: Set maximum quantity of Agenet usage.
    max_agent_value = resource2.Body('max_agent_value')
    #: Contactable mail address.
    mailaddress = resource2.Body('mailaddress')
    #: This value is used for language of Deep Security Manager.
    #: ja: Japanese, en: English.
    dsm_lang = resource2.Body('dsm_lang')
    #: Set "Asia/Tokyo" for JST or "Etc/GMT" for UTC.
    time_zone = resource2.Body('time_zone')
    #: Messages are displayed in Japanese or English depending on this value.
    #: ja: Japanese, en: English. Default value is "en".
    locale = resource2.Body('locale')
    #: This value indicates normal or abnormal. 1:normal, 2:abnormal.
    code = resource2.Body('code', alternate_id=True)
    #: This message is shown when error has occurred.
    message = resource2.Body('message')
    #: Identification ID of Service Order.
    soid = resource2.Body('soId')
    #: This value indicates normal or abnormal. 1:normal, 2:abnormal.
    status = resource2.Body('status')
    #: Percentage of Service Order Progress Status.
    progress_rate = resource2.Body('progressRate')
    #: Region tenant you specified belongs to.
    region = resource2.Body('region')
    #: Tenant Name.
    tenant_name = resource2.Body('tenant_name')
    #: Description for this tenant.
    tenant_description = resource2.Body('tenant_description')
    #: Contract ID which this tenant belongs to.
    contract_id = resource2.Body('contract_id')
    #: Customer Name.
    customer_name = resource2.Body('customer_name')
    #: Internal Use. (true: Already applied, false: Not applied.)
    tenant_flg = resource2.Body('tenant_flg')

    def get_order_status(self, session, soid, locale=None):
        tenant_id = session.get_project_id()
        uri = '/API/ScreenEventHBSOrderProgressRate?tenant_id=%s&soid=%s' \
              % (tenant_id, soid)
        if locale is not None:
            uri += '&locale=%s' % locale
        headers = {'Content-Type': 'application/json'}
        resp = session.get(uri, endpoint_filter=self.service, headers=headers)
        self._translate_response(resp, has_body=True)
        return self

    def get_order_info(self, session, locale=None):
        tenant_id = session.get_project_id()
        uri = '/API/ScreenEventHBSOrderInfoGet?tenant_id=%s' % tenant_id
        if locale is not None:
            uri += '&locale=%s' % locale
        headers = {'Content-Type': 'application/json'}
        resp = session.get(uri, endpoint_filter=self.service, headers=headers)
        self._translate_response(resp, has_body=True)
        return self

    def update(self, session, **body):
        uri = self.base_path
        resp = session.post(uri, endpoint_filter=self.service, json=body)
        self._translate_response(resp, has_body=True)
        return self

    def delete(self, session, body, locale=None):
        uri = self.base_path
        resp = session.post(uri, endpoint_filter=self.service, json=body)
        self._translate_response(resp, has_body=True)
        return self



"""
Exception definitions for Virtual Network Appliance
"""
import json
from ecl import exceptions


class HttpException(exceptions.HttpException):

    def _get_exception_message(self, message=None):
        try:
            content = json.loads(self.response._content)

            # API-GW
            if 'fault' in content and 'faultstring' in content['fault']:
                return content['fault']['faultstring']

            # IAM error case.
            if content.get('errorMessage'):
                return content[self.api_error_key]

            # In VNA API, we need to handle both "cause" and "message" key
            # as API error.
            k = content.keys()
            if 'message' in k and 'cause' not in k:
                return content['message']
            if 'cause' in k and 'message' not in k:
                return content['cause']
        except:
            pass

        if message:
            return message

        # In case parse failed.
        return 'Unknown Exception'


class NotFoundException(HttpException):
    pass



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
The :class:`~ecl.connection.Connection` class is the primary interface
to the Python SDK it maintains a context for a connection to a cloud provider.
The connection has an attribute to access each supported service.  The service
attributes are created dynamically based on user profiles and the service
catalog.

Examples
--------

At a minimum, the :class:`~ecl.connection.Connection` class needs to be
created with an authenticator or the parameters to build one.

Create a connection
~~~~~~~~~~~~~~~~~~~

The following example constructor uses the identity authenticator using
username and password.  The default settings for the transport are used
by this connection.::

    from ecl import connection
    auth_args = {
        'auth_url': 'http://172.20.1.108:5000/v3',
        'project_name': 'admin',
        'username': 'admin',
        'password': 'admin',
    }
    conn = connection.Connection(**auth_args)

List
~~~~

Services are accessed through an attribute named after the service.  A list
of all the projects is retrieved in this manner::

    projects = conn.identity.list_projects()

Find or create
~~~~~~~~~~~~~~
If you wanted to make sure you had a network named 'jenkins', you would first
try to find it and if that fails, you would create it::

    network = conn.network.find_network("jenkins")
    if network is None:
        network = conn.network.create_network({"name": "jenkins"})

"""
import logging
import sys

from keystoneauth1.loading import base as ksa_loader
import os_client_config

from ecl import profile as _profile
from ecl import proxy
from ecl import proxy2
from ecl import session as _session
from ecl import utils

_logger = logging.getLogger(__name__)
_logger.addHandler(logging.StreamHandler())


def from_config(cloud_name=None, cloud_config=None, options=None):
    """Create a Connection using os-client-config

    :param str cloud_name: Use the `cloud_name` configuration details when
                           creating the Connection instance.
    :param cloud_config: An instance of
                         `os_client_config.config.OpenStackConfig`
                         as returned from the os-client-config library.
                         If no `config` is provided,
                         `os_client_config.OpenStackConfig` will be called,
                         and the provided `cloud_name` will be used in
                         determining which cloud's configuration details
                         will be used in creation of the
                         `Connection` instance.
    :param options: An argparse Namespace object; allows direct passing
                    in of argparse options to be added to the cloud config.
                    This value is passed to the `argparse` argument of
                    `os_client_config.config.OpenStackConfig.get_one_cloud`.

    :rtype: :class:`~ecl.connection.Connection`
    """
    # TODO(thowe): I proposed that service name defaults to None in OCC
    defaults = {}
    prof = _profile.Profile()
    services = [service.service_type for service in prof.get_services()]
    for service in services:
        defaults[service + '_service_name'] = None
    # TODO(thowe): default is 2 which turns into v2 which doesn't work
    # this stuff needs to be fixed where we keep version and path separated.
    defaults['network_api_version'] = 'v2.0'
    if cloud_config is None:
        occ = os_client_config.OpenStackConfig(override_defaults=defaults)
        cloud_config = occ.get_one_cloud(cloud=cloud_name, argparse=options)

    if cloud_config.debug:
        utils.enable_logging(True, stream=sys.stdout)

    # TODO(mordred) we need to add service_type setting to eclsdk.
    # Some clouds have type overridden as well as name.
    services = [service.service_type for service in prof.get_services()]
    for service in cloud_config.get_services():
        if service in services:
            version = cloud_config.get_api_version(service)
            if version:
                version = str(version)
                if not version.startswith("v"):
                    version = "v" + version
                prof.set_version(service, version)
            name = cloud_config.get_service_name(service)
            if name:
                prof.set_name(service, name)
            interface = cloud_config.get_interface(service)
            if interface:
                prof.set_interface(service, interface)

    region = cloud_config.get_region_name(service)
    if region:
        for service in services:
            prof.set_region(service, region)

    # Auth
    auth = cloud_config.config['auth']
    # TODO(thowe) We should be using auth_type
    auth['auth_plugin'] = cloud_config.config['auth_type']
    if 'cacert' in auth:
        auth['verify'] = auth.pop('cacert')
    if 'cacert' in cloud_config.config:
        auth['verify'] = cloud_config.config['cacert']
    insecure = cloud_config.config.get('insecure', False)
    if insecure:
        auth['verify'] = False

    cert = cloud_config.config.get('cert')
    if cert:
        key = cloud_config.config.get('key')
        auth['cert'] = (cert, key) if key else cert

    return Connection(profile=prof, **auth)


class Connection(object):

    def __init__(self, session=None, authenticator=None, profile=None,
                 verify=True, cert=None, user_agent=None,
                 auth_plugin="password", timeout=None,
                 **auth_args):
        """Create a context for a connection to a cloud provider.

        A connection needs a transport and an authenticator.  The user may pass
        in a transport and authenticator they want to use or they may pass in
        the parameters to create a transport and authenticator.  The connection
        creates a
        :class:`~ecl.session.Session` which uses the profile
        and authenticator to perform HTTP requests.

        :param session: A session object compatible with
            :class:`~ecl.session.Session`.
        :type session: :class:`~ecl.session.Session`
        :param authenticator: An authenticator derived from the base
            authenticator plugin that was previously created.  Two common
            authentication identity plugins are
            :class:`identity_v2 <ecl.auth.identity.v2.Auth>` and
            :class:`identity_v3 <ecl.auth.identity.v3.Auth>`.
            If this parameter is not passed in, the connection will create an
            authenticator.
        :type authenticator: :class:`~ecl.auth.base.BaseAuthPlugin`
        :param profile: If the user has any special profiles such as the
            service name, region, version or interface, they may be provided
            in the profile object.  If no profiles are provided, the
            services that appear first in the service catalog will be used.
        :type profile: :class:`~ecl.profile.Profile`
        :param bool verify: If a transport is not provided to the connection,
            this parameter will be used to create a transport.  If ``verify``
            is set to true, which is the default, the SSL cert will be
            verified.  It can also be set to a CA_BUNDLE path.
        :param cert: If a transport is not provided to the connection then this
            parameter will be used to create a transport. `cert` allows to
            provide a client certificate file path or a tuple with client
            certificate and key paths.
        :type cert: str or tuple
        :param str user_agent: If a transport is not provided to the
            connection, this parameter will be used when creating a transport.
            The value given here will be prepended to the default, which is
            specified in :attr:`~ecl.transport.USER_AGENT`.
            The resulting ``user_agent`` value is used for the ``User-Agent``
            HTTP header.
        :param str auth_plugin: The name of authentication plugin to use.
            The default value is ``password``.
        :param float timeout: A timeout to pass to requests. This should be a
            numerical value indicating some amount (or fraction)
            of seconds or 0 for no timeout. (optional, defaults to 0)
        :param auth_args: The rest of the parameters provided are assumed to be
            authentication arguments that are used by the authentication
            plugin.
        """
        self.authenticator = self._create_authenticator(authenticator,
                                                        auth_plugin,
                                                        **auth_args)
        self.profile = profile if profile else _profile.Profile()
        self.session = session if session else _session.Session(
            self.profile, auth=self.authenticator, verify=verify, cert=cert,
            user_agent=user_agent, timeout=timeout)
        self._open()

    def _create_authenticator(self, authenticator, auth_plugin, **args):
        if authenticator:
            return authenticator
        # TODO(thowe): Jamie was suggesting we should support other
        #              ways of loading the plugin
        loader = ksa_loader.get_plugin_loader(auth_plugin)
        load_args = {}
        for opt in loader.get_options():
            if args.get(opt.dest):
                load_args[opt.dest] = args[opt.dest]
        return loader.load_from_options(**load_args)

    def _open(self):
        """Open the connection.

        NOTE(thowe): Have this set up some lazy loader instead.
        """
        for service in self.profile.get_services():
            self._load(service)

    def _load(self, service):
        attr_name = service.get_service_module()
        module = service.get_module() + "._proxy"
        try:
            __import__(module)
            proxy_class = getattr(sys.modules[module], "Proxy")
            if not (issubclass(proxy_class, proxy.BaseProxy) or
                    issubclass(proxy_class, proxy2.BaseProxy)):
                raise TypeError("%s.Proxy must inherit from BaseProxy" %
                                proxy_class.__module__)
            setattr(self, attr_name, proxy_class(self.session))
        except Exception as e:
            _logger.warn("Unable to load %s: %s" % (module, e))

    def authorize(self):
        """Authorize this Connection

        **NOTE**: This method is optional. When an application makes a call
                  to any OpenStack service, this method allows you to request
                  a token manually before attempting to do anything else.

        :returns: A string token.

        :raises:`~ecl.exceptions.HttpException` if the authorization
                fails due to reasons like the credentials provided are unable
                to be authorized or the `auth_plugin` argument is missing,
                etc.
        """
        headers = self.session.get_auth_headers()

        return headers.get('X-Auth-Token') if headers else None



# -*- coding: utf-8 -*-

from ecl.network import network_service
from ecl.network.v2.base import NetworkBaseResource
from ecl import resource2
from ecl import exceptions


class GCPService(NetworkBaseResource):

    resource_key = "gcp_service"
    resources_key = "gcp_services"
    service = network_service.NetworkService("v2.0")
    base_path = '/' + service.version + '/gcp_services'

    allow_list = True
    allow_get = True

    #: Description of the GCP Service resource.
    description = resource2.Body("description")
    #: Unique ID of the GCP Service resource.
    id = resource2.Body("id")
    #: Name of zone.
    zone = resource2.Body("zone")
    #: Name of the GCP Service resource.
    name = resource2.Body("name")


class GCPGateway(NetworkBaseResource):

    resource_key = "gcp_gateway"
    resources_key = "gcp_gateways"
    service = network_service.NetworkService("v2.0")
    base_path = '/' + service.version + '/gcp_gateways'

    allow_list = True
    allow_get = True

    #: Description of the GCP Gateway resource.
    description = resource2.Body("description")
    #: Unique ID of the GCP Gateway resource.
    id = resource2.Body("id")
    #: GCP Service instantiated by this Gateway.
    gcp_service_id = resource2.Body("gcp_service_id")
    #: Name of the GCP Gateway resource.
    name = resource2.Body("name")
    #: Quality of Service options selected for this Gateway.
    qos_option_id = resource2.Body("qos_option_id")
    #: The GCP Gateway status.
    status = resource2.Body("status")
    #: Tenant ID of the owner (UUID)
    tenant_id = resource2.Body("tenant_id")


class GCPInterface(NetworkBaseResource):

    resource_key = "gcp_interface"
    resources_key = "gcp_interfaces"
    service = network_service.NetworkService("v2.0")
    base_path = '/' + service.version + '/gcp_interfaces'

    allow_list = True
    allow_get = True

    #: GCP Gateway to which this interface is connected.
    gcp_gw_id = resource2.Body("gcp_gw_id")
    #: Description of the GCP Interface resource.
    description = resource2.Body("description")
    #: Unique ID of the GCP Interface resource.
    id = resource2.Body("id")
    #: Name of the GCP Interface resource
    name = resource2.Body("name")
    #: The GCP Interface status.
    status = resource2.Body("status")
    #: Tenant ID of the owner (UUID)
    tenant_id = resource2.Body("tenant_id")
    #: Primary router uplink ip configuration.
    primary = resource2.Body("primary")
    #: Secondary router uplink ip configuration.
    secondary = resource2.Body("secondary")



# -*- coding: utf-8 -*-

from ecl.network import network_service
from ecl import resource2


class ReservedAddress(resource2.Resource):
    resource_key = 'reserve_address'
    resources_key = 'reserve_addresses'
    service = network_service.NetworkService("v2.0")
    base_path = '/' + service.version + '/reserve_addresses'

    _query_mapping = resource2.QueryParameters(
        "subnets", "id", "destination", "sort_key", "sort_dir",
    )

    # capabilities
    allow_create = False
    allow_get = True
    allow_update = False
    allow_delete = False
    allow_list = True

    # Properties
    #: id of the reserved address
    id = resource2.Body("id")
    #: subnets cidr of the reserved address
    subnets = resource2.Body("subnets")
    #: tenant id that reserved address belong to
    tenant_id = resource2.Body("tenant_id")



# -*- coding: utf-8 -*-

from ecl.tests.functional import base


class TestVPNService(base.BaseFunctionalTest):

    @classmethod
    def setUpClass(cls):
        super(TestVPNService, cls).setUpClass()
        cls.one_vpn_service = None
        vpn_services = cls.conn.network.vpn_services()
        if vpn_services and len(vpn_services) > 0:
            cls.one_vpn_service = vpn_services[0]

    def test_get(self):
        sot = self.conn.network.get_vpn_service(self.one_vpn_service.id)
        self.assertEqual(self.one_vpn_service.id, sot.id)


class TestVPNGateway(base.BaseFunctionalTest):

    @classmethod
    def setUpClass(cls):
        super(TestVPNGateway, cls).setUpClass()
        cls.one_vpn_gateway = None
        vpn_gws = cls.conn.network.vpn_gateways()
        if vpn_gws and len(vpn_gws) > 0:
            cls.one_vpn_gateway = vpn_gws[0]

    def test_get(self):
        sot = self.conn.network.get_vpn_gateway(self.one_vpn_gateway.id)
        self.assertEqual(self.one_vpn_gateway.id, sot.id)

    def find_get_by_id(self):
        sot = self.conn.network.find_vpn_gateway(self.one_vpn_gateway.id)
        self.assertEqual(self.one_vpn_gateway.id, sot.id)


class TestVPNInterface(base.BaseFunctionalTest):

    @classmethod
    def setUpClass(cls):
        super(TestVPNInterface, cls).setUpClass()
        cls.one_vpn_interface = None
        vpn_ifs = cls.conn.network.vpn_interfaces()
        if vpn_ifs and len(vpn_ifs) > 0:
            cls.one_vpn_interface = vpn_ifs[0]

    def test_get(self):
        sot = self.conn.network.get_vpn_interface(self.one_vpn_interface.id)
        self.assertEqual(self.one_vpn_interface.id, sot.id)


# -*- coding: utf-8 -*-


import six

from ecl import exceptions
from ecl.tests.functional import base


class TestOperation(base.BaseFunctionalTest):

    @classmethod
    def setUpClass(cls):
        super(TestOperation, cls).setUpClass()
        mcics = list(cls.conn.connectivity.mcics())
        cls.one_mcic = None
        cls.one_operation = None
        cls.one_cic = None
        if mcics and len(mcics) > 0:
            cls.one_mcic = mcics[0]
        if cls.one_mcic is not None:
            cics = list(cls.conn.connectivity.cics(cls.one_mcic.mcic_id))
            if cics and len(cics) > 0:
                cls.one_cic = cics[0]
            operations = list(cls.conn.connectivity.operations(mcic_id=cls.one_mcic.mcic_id))
            if operations and len(operations) > 0:
                cls.one_operation = operations[0]

    def test_operations_by_mcic(self):
        if self.one_mcic is None:
            return
        operations = list(self.conn.connectivity.operations(self.one_mcic.mcic_id))
        self.assertGreaterEqual(len(operations), 0)

        for operation in operations:
            # print operation.to_dict()["mcic_id"]
            self.assertIsInstance(operation.operation_id, six.string_types)
            self.assertIsInstance(operation.operation_status, six.string_types)
            self.assertIsInstance(operation.operation_type, six.string_types)
            self.assertIsInstance(operation.operation_body, dict)
            self.assertEqual(operation.mcic_id, self.one_mcic.mcic_id)
            self.assertIsInstance(operation.user_id, six.string_types)
            self.assertIsInstance(operation.user_name, six.string_types)
            if operation.reciept_date:
                self.assertIsInstance(operation.reciept_date, six.string_types)
            self.assertIsInstance(operation.error_messages, list)

    def test_operations_by_cic(self):
        if self.one_cic is None:
            return
        cic_operations = self.conn.connectivity.operations(cic_id=self.one_cic.cic_id)
        for operation in cic_operations:
            # print operation.to_dict()["cic_id"]
            self.assertIsInstance(operation.operation_id, six.string_types)
            self.assertIsInstance(operation.operation_status, six.string_types)
            self.assertIsInstance(operation.operation_type, six.string_types)
            self.assertIsInstance(operation.operation_body, dict)
            self.assertEqual(operation.cic_id, self.one_cic.cic_id)
            self.assertIsInstance(operation.user_id, six.string_types)
            self.assertIsInstance(operation.user_name, six.string_types)
            if operation.reciept_date:
                self.assertIsInstance(operation.reciept_date, six.string_types)
            self.assertIsInstance(operation.error_messages, list)

    def test_get_operation_by_id(self):
        if self.one_operation is None:
            return
        operation = self.conn.connectivity.get_operation(self.one_operation.operation_id)
        # print operation.to_dict()
        self.assertEqual(operation.operation_id, self.one_operation.operation_id)
        self.assertEqual(operation.operation_status, self.one_operation.operation_status)
        self.assertEqual(operation.operation_type, self.one_operation.operation_type)
        self.assertEqual(operation.operation_body, self.one_operation.operation_body)
        self.assertEqual(operation.user_id, self.one_operation.user_id)
        self.assertEqual(operation.user_name, self.one_operation.user_name)
        self.assertEqual(operation.reciept_date, self.one_operation.reciept_date)
        self.assertEqual(operation.error_messages, self.one_operation.error_messages)



# -*- coding: utf-8 -*-

from ecl.block_store import block_store_service
from ecl import resource


class Metadata(resource.Resource):
    resource_key = "metadata"
    resources_key = "metadata"
    base_path = "/volumes/%(volume_id)s/metadata"
    service = block_store_service.BlockStoreService()

    # capabilities
    allow_list = True
    allow_create = True
    allow_update = True


    def update(self, session, volume_id, **metadata):
        uri = self.base_path % {"volume_id": volume_id}
        body = {"metadata": metadata}
        resp = session.post(
            uri,
            endpoint_filter=self.service,
            json=body,
            headers={"Accept": ""}
        )

        meta = resp.json().get('metadata')
        for key in meta.keys():
            self.__setattr__(key, meta[key])

        return self

    def create(self, session, volume_id, **metadata):
        uri = self.base_path % {"volume_id": volume_id}
        body = {"metadata": metadata}
        resp = session.put(
            uri,
            endpoint_filter=self.service,
            json=body,
            headers={"Accept": ""}
        )

        meta = resp.json().get('metadata')
        for key in meta.keys():
            self.__setattr__(key, meta[key])

        return self



# -*- coding: utf-8 -*-


from ecl import resource2


class NetworkBaseResource(resource2.Resource):

    @property
    def name_or_id(self):
        try:
            return (self._body.get('name') or
                    '(%s)' % self._body['id'][:13])
        except KeyError:
            pass

    def set_id_as_name_if_empty(self, length=8):
        try:
            if not self._body.get('name'):
                id = self._body['id']
                if length:
                    id = id[:length]
                setattr(self, 'name', '(%s)' % id)
        except KeyError:
            pass


    #: The ID of the project this resource is associated with.
    tenant_id = resource2.Body('tenant_id')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from ecl.identity import identity_service
from ecl import resource


class Extension(resource.Resource):
    resource_key = 'extension'
    resources_key = 'extensions'
    base_path = '/extensions'
    service = identity_service.IdentityService()

    # capabilities
    allow_list = True

    # Properties
    #: A unique identifier, which will be used for accessing the extension
    #: through a dedicated url ``/extensions/*alias*``. The extension
    #: alias uniquely identifies an extension and is prefixed by a vendor
    #: identifier. *Type: string*
    alias = resource.prop('alias')
    #: A description of the extension. *Type: string*
    description = resource.prop('description')
    #: Links to the documentation in various format. *Type: string*
    links = resource.prop('links')
    #: The name of the extension. *Type: string*
    name = resource.prop('name')
    #: The second unique identifier of the extension after the alias.
    #: It is usually a URL which will be used. Example:
    #: "http://docs.ecl.org/identity/api/ext/s3tokens/v1.0"
    #: *Type: string*
    namespace = resource.prop('namespace')
    #: The last time the extension has been modified (update date).
    updated_at = resource.prop('updated')

    @classmethod
    def list(cls, session, **params):
        resp = session.get(cls.base_path, endpoint_filter=cls.service,
                           params=params)
        resp = resp.json()
        for data in resp[cls.resources_key]['values']:
            yield cls.existing(**data)



# -*- coding: utf-8 -*-

import testtools

from ecl.provider_connectivity.v1 import address_assignment


IDENTIFIER = 'IDENTIFIER'
REQUEST_ID = 'REQUEST_ID'
BASIC_EXAMPLE = {
    'tenant_connection_id': IDENTIFIER,
    'tenant_connection_request_id': REQUEST_ID,
    'network_id': 'network_id',
    'mac_address': 'mac_address'
}


class TestTenantConnection(testtools.TestCase):
    def test_basic(self):
        sot = address_assignment.AddressAssignment(REQUEST_ID)
        self.assertEqual('address_assignments', sot.resources_key)
        self.assertEqual('address_assignment', sot.resource_key)
        self.assertEqual('/v2.0/tenant_connection_requests/'
                         '%(tenant_connection_request_id)s/address_assignment', sot.base_path)
        self.assertEqual('provider-connectivity', sot.service.service_type)
        self.assertTrue(sot.allow_list)
        self.assertFalse(sot.allow_create)
        self.assertFalse(sot.allow_get)
        self.assertFalse(sot.allow_delete)
        self.assertFalse(sot.allow_update)

    def test_make_basic(self):
        sot = address_assignment.AddressAssignment(**BASIC_EXAMPLE)
        self.assertEqual(BASIC_EXAMPLE['tenant_connection_id'], sot.tenant_connection_id)
        self.assertEqual(BASIC_EXAMPLE['tenant_connection_request_id'], sot.tenant_connection_request_id)
        self.assertEqual(BASIC_EXAMPLE['network_id'], sot.network_id)
        self.assertEqual(BASIC_EXAMPLE['mac_address'], sot.mac_address)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import six

from ecl import resource
from ecl.telemetry import telemetry_service


class Capability(resource.Resource):
    """.. caution:: This API is a work in progress and is subject to change."""
    resource_key = 'capability'
    resources_key = 'capabilities'
    base_path = '/capabilities'
    service = telemetry_service.TelemetryService()

    # Supported Operations
    allow_list = True

    # Properties
    is_enabled = resource.prop('enabled', type=bool)

    @classmethod
    def list(cls, session, limit=None, marker=None, path_args=None,
             paginated=False, **params):
        resp = session.get(cls.base_path, endpoint_filter=cls.service,
                           params=params)
        resp = resp.json()
        for key, value in six.iteritems(resp['api']):
            yield cls.existing(id=key, enabled=value)



# Copyright 2010-2011 OpenStack Foundation
# Copyright (c) 2013 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import mock


class FakeTransport(mock.Mock):
    RESPONSE = mock.Mock('200 OK')

    def __init__(self):
        super(FakeTransport, self).__init__()
        self.request = mock.Mock()
        self.request.return_value = self.RESPONSE


class FakeAuthenticator(mock.Mock):
    TOKEN = 'fake_token'
    ENDPOINT = 'http://www.example.com/endpoint'

    def __init__(self):
        super(FakeAuthenticator, self).__init__()
        self.get_token = mock.Mock()
        self.get_token.return_value = self.TOKEN
        self.get_endpoint = mock.Mock()
        self.get_endpoint.return_value = self.ENDPOINT



# -*- coding: utf-8 -*-

import six

from ecl.tests.functional import base


class TestVolume(base.BaseFunctionalTest):

    @classmethod
    def setUpClass(cls):
        super(TestVolume, cls).setUpClass()
        vols = cls.conn.compute.volumes()
        cls.one_volume = None
        if vols and len(vols):
            cls.one_volume = vols[0]

    def _test_volumes(self):
        vols = self.conn.compute.volumes()
        for vol in vols:
            self.assertIsInstance(vol.id, six.string_types)

    def _test_get_volumes(self):
        if self.one_volume is None:
            return
        vol = self.conn.compute.get_volume(self.one_volume.id)
        self.assertEqual(vol.id, self.one_volume.id)

    def _test_create(self):
        test_name = "test-volume-sdk"
        vol = self.conn.compute.create_volume(15, name=test_name)
        self.assertEqual(vol.name, test_name)

    def test_delete(self):
        test_name = "test-volume-sdk"
        vols = self.conn.compute.volumes()
        find_vol = None
        for vol in vols:
            if vol.name == test_name:
                find_vol = vol
                break
        if find_vol:
            ret = self.conn.compute.delete_volume(find_vol.id)
            self.assertIsNone(ret)



# -*- coding: utf-8 -*-

from ecl.tests.functional import base
from ecl.network.v2 import interdc as _interdc


class TestInterDCService(base.BaseFunctionalTest):

    @classmethod
    def setUpClass(cls):
        super(TestInterDCService, cls).setUpClass()
        cls.one_idc_service = None
        idc_services = cls.conn.network.interdc_services()
        if idc_services and len(idc_services) > 0:
            cls.one_idc_service = idc_services[0]

    def test_get(self):
        sot = self.conn.network.get_interdc_service(self.one_idc_service.id)
        self.assertEqual(self.one_idc_service.id, sot.id)


class TestInterDCGateway(base.BaseFunctionalTest):

    @classmethod
    def setUpClass(cls):
        super(TestInterDCGateway, cls).setUpClass()
        cls.one_idc_gateway = None
        idc_gws = cls.conn.network.interdc_gateways()
        if idc_gws and len(idc_gws) > 0:
            cls.one_idc_gateway = idc_gws[0]

    def test_get(self):
        sot = self.conn.network.get_interdc_gateway(self.one_idc_gateway.id)
        self.assertEqual(self.one_idc_gateway.id, sot.id)

    def test_find_by_id(self):
        sot = set.conn.network.find_interdc_gateway(
            self.one_idc_gateway.id
        )
        self.assertEqual(self.one_idc_gateway.id, sot.id)


class TestInterDCInterface(base.BaseFunctionalTest):

    @classmethod
    def prepare_create(cls):
        interdc_gws = cls.conn.network.interdc_gateways()
        if interdc_gws and len(interdc_gws) > 0:
            cls.one_idc_gateway = interdc_gws[0]
        network = cls.conn.network.networks()
        for nw in network:
            if nw.name == "test-network":
                cls.one_network = nw
                break

    @classmethod
    def setUpClass(cls):
        super(TestInterDCInterface, cls).setUpClass()
        cls.one_idc_interface = None
        cls.one_idc_gateway = None
        cls.one_idc_gateway_name = "N000001996_V15000001"
        cls.one_idc_interface_name = "interdc_interface_for_tenant-sdpgui01"
        cls.one_network = None

        idc_ifs = cls.conn.network.interdc_interfaces()
        for idcif in idc_ifs:
            if idcif.name == cls.one_idc_interface_name:
                cls.one_idc_interface = idcif
        if cls.one_idc_interface is None:
            cls.prepare_create()
            cls.one_idc_interface = cls.conn.network.create_interdc_interface(
                cls.one_idc_gateway.id,
                "xxx", "xxx", "xxx", "xxx", 1,
                name=cls.one_idc_interface_name, )

    def test_get(self):
        sot = self.conn.network.get_interdc_interface(self.one_idc_interface.id)
        self.assertEqual(self.one_idc_interface.id, sot.id)

    def test_create(self):
        if self.one_idc_gateway is None or self.one_network is None:
            self.prepare_create()
        self.one_idc_interface = self.conn.network.create_interdc_interface(
            self.one_idc_gateway.id,
            "xxx", "xxx", "xxx", "xxx", 1,
            name=self.one_idc_interface_name, )
        self.assertIsInstance(self.one_idc_interface, _interdc.InterDCInterface)

    def test_update(self):
        sot = self.conn.network.update_interdc_interface(self.one_idc_interface.id,
                                                         description="xxx")
        self.assertIsInstance(sot, _interdc.InterDCInterface)

    def test_delete(self):
        sot = self.conn.network.delete_interdc_interface(self.one_idc_interface.id)
        # self.assertIsInstance(sot, _interdc.InterDCInterface)


# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import  testtools

from ecl.sss.v1 import contract

BASIC_EXAMPLE = {
    "contract_id": "econXXXXXXXXXX",
    "status": "enable",
    "customer_number": "N1234567890",
    "channel_name": "test_channel",
    "channel_id": "chXXXXX",
    "management_channel": False,
    "contract_owner_user_id": "ecidXXXXXX",
    "start_time": "2016-03-04 12:24:50",
    "end_time": "",
    "login_integration": "",
    "sss_endpoint": "https://xxxxxxxxxx/api",
    "keystone_endpoint": "https://keystone-xxxxxxxxxx:443",
    "company_code": "C1234567890",
    "glass_customer_id": "",
    "glass_user_id": "",
    "owner_contract_id": "econxxxxxxxxxx",
    "charge_status": "xxxxxxxxx",
    "cycle_start_time": "yyyy-mm-dd hh:mm:ss",
    "cycle_end_time": "yyyy-mm-dd hh:mm:ss",
    "charge_data": {},
    "notify_password": False,
}

class TestRole(testtools.TestCase):

    def test_role(self):
        sot = contract.Contract()
        self.assertEqual(None, sot.resources_key)
        self.assertEqual(None, sot.resource_key)
        self.assertEqual("/contracts", sot.base_path)
        self.assertEqual('sss', sot.service.service_type)

        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_get)
        self.assertFalse(sot.allow_update)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)
        self.assertFalse(sot.put_create)

    def test_make_role(self):
        sot = contract.Contract(**BASIC_EXAMPLE)
        self.assertEqual(BASIC_EXAMPLE['contract_id'], sot.contract_id)
        self.assertEqual(BASIC_EXAMPLE['status'], sot.status)
        self.assertEqual(BASIC_EXAMPLE['customer_number'], sot.customer_number)
        self.assertEqual(BASIC_EXAMPLE['channel_name'], sot.channel_name)
        self.assertEqual(BASIC_EXAMPLE['channel_id'], sot.channel_id)
        self.assertEqual(BASIC_EXAMPLE['notify_password'], sot.notify_password)
        self.assertEqual(BASIC_EXAMPLE['management_channel'], sot.management_channel)
        self.assertEqual(BASIC_EXAMPLE['contract_owner_user_id'], sot.contract_owner_user_id)
        self.assertEqual(BASIC_EXAMPLE['contract_owner_user_id'], sot.contract_owner_user_id)
        self.assertEqual(BASIC_EXAMPLE['start_time'], sot.start_time)
        self.assertEqual(BASIC_EXAMPLE['end_time'], sot.end_time)
        self.assertEqual(BASIC_EXAMPLE['login_integration'], sot.login_integration)
        self.assertEqual(BASIC_EXAMPLE['sss_endpoint'], sot.sss_endpoint)
        self.assertEqual(BASIC_EXAMPLE['keystone_endpoint'], sot.keystone_endpoint)
        self.assertEqual(BASIC_EXAMPLE['company_code'], sot.company_code)
        self.assertEqual(BASIC_EXAMPLE['glass_customer_id'], sot.glass_customer_id)
        self.assertEqual(BASIC_EXAMPLE['glass_user_id'], sot.glass_user_id)
        self.assertEqual(BASIC_EXAMPLE['owner_contract_id'], sot.owner_contract_id)
        self.assertEqual(BASIC_EXAMPLE['charge_status'], sot.charge_status)
        self.assertEqual(BASIC_EXAMPLE['cycle_start_time'], sot.cycle_start_time)
        self.assertEqual(BASIC_EXAMPLE['cycle_end_time'], sot.cycle_end_time)
        self.assertEqual(BASIC_EXAMPLE['charge_data'], sot.charge_data)
        self.assertEqual(BASIC_EXAMPLE['notify_password'], sot.notify_password)



# -*- coding: utf-8 -*-

import six

from ecl.tests.functional import base


class TestQuota(base.BaseFunctionalTest):

    @classmethod
    def setUpClass(cls):
        super(TestQuota, cls).setUpClass()

    def test_quota(self):
        tenant_id = "3ac7407ad7ab4e2fa0665df71a4886e4"
        quota = self.conn.block_store.get_quota(tenant_id)

        self.assertEqual(quota.id, tenant_id)

    def test_get_detail_quota(self):
        tenant_id = "3ac7407ad7ab4e2fa0665df71a4886e4"
        user_id = tenant_id
        quota = self.conn.block_store.get_quota_detail(tenant_id, user_id)

        self.assertIsInstance(quota.id, six.string_types)

    def test_get_default_quota(self):
        tenant_id = "3ac7407ad7ab4e2fa0665df71a4886e4"
        info = self.conn.block_store.get_default_quota(tenant_id)

        self.assertIsInstance(info.id, six.string_types)



# -*- coding: utf-8 -*-

import six

from ecl.tests.functional import base


class TestQuota(base.BaseFunctionalTest):

    @classmethod
    def setUpClass(cls):
        super(TestQuota, cls).setUpClass()

    def test_quota(self):
        tenant_id = "3ac7407ad7ab4e2fa0665df71a4886e4"
        quota = self.conn.compute.get_quota(tenant_id)

        self.assertIsInstance(quota.cores, int)
        self.assertIsInstance(quota.fixed_ips, int)
        self.assertIsInstance(quota.id, six.string_types)
        self.assertIsInstance(quota.injected_file_content_bytes, int)
        self.assertIsInstance(quota.injected_file_path_bytes, int)
        self.assertIsInstance(quota.instances, int)
        self.assertIsInstance(quota.key_pairs, int)
        self.assertIsInstance(quota.metadata_items, int)
        self.assertIsInstance(quota.ram, int)
        self.assertIsInstance(quota.security_group_rules, int)
        self.assertIsInstance(quota.security_groups, int)
        self.assertIsInstance(quota.server_group_members, int)
        self.assertIsInstance(quota.server_groups, int)

    def test_get_default_quota(self):
        tenant_id = "3ac7407ad7ab4e2fa0665df71a4886e4"
        quota = self.conn.compute.get_default_quota(tenant_id)

        self.assertIsInstance(quota.cores, int)
        self.assertIsInstance(quota.fixed_ips, int)
        self.assertIsInstance(quota.id, six.string_types)
        self.assertIsInstance(quota.injected_file_content_bytes, int)
        self.assertIsInstance(quota.injected_file_path_bytes, int)
        self.assertIsInstance(quota.instances, int)
        self.assertIsInstance(quota.key_pairs, int)
        self.assertIsInstance(quota.metadata_items, int)
        self.assertIsInstance(quota.ram, int)
        self.assertIsInstance(quota.security_group_rules, int)
        self.assertIsInstance(quota.security_groups, int)
        self.assertIsInstance(quota.server_group_members, int)
        self.assertIsInstance(quota.server_groups, int)

    def test_get_tenant_usage(self):
        tenant_id = "3ac7407ad7ab4e2fa0665df71a4886e4"
        info = self.conn.compute.get_tenant_usage(tenant_id)

        self.assertIsInstance(info.start, six.string_types)
        self.assertIsInstance(info.stop, six.string_types)
        self.assertIsInstance(info.tenant_id, six.string_types)
